<?php
/**
 * General functions
 *
 * Contains globally available tool functions. It is included in every
 * page and sets up some defaults like error reporting, environment
 * setups and config loading
 *
 * @todo    Encapsulate httpClient and Cache as separate classes
 *
 * @package Core
 * @author  Andreas Goetz   <cpuidle@gmx.de>
 * @author  Andreas Gohr    <a.gohr@web.de>
 * @author  Chinamann       <chinamann@users.sourceforge.net>
 * @version $Id: functions.php,v 1.60 2006/01/01 13:15:32 andig2 Exp $
 */

// add pwd to include_path
ini_set('include_path', '.:'.ini_get('include_path'));

/* This is needed to make sure we have our very own namespace without
   interfering Environment variables */
foreach (array_keys($_ENV) as $key)
{
    unset($GLOBALS[$key]);
}

require_once './config.inc.php';          // global const CONFIG_FILE is not yet defined at this point
require_once './core/constants.php';
require_once './core/template.php';
require_once './core/cache.php';
require_once './core/compatibility.php';
require_once './smarty/Smarty.class.php';

/* --------------------------------------------------------------------*/
// Set up some defaults

error_reporting($config['debug'] ? E_ALL ^ E_NOTICE : E_ERROR + E_PARSE);

if (get_magic_quotes_gpc())
{
	if (!empty($_REQUEST)) remove_magic_quotes($_REQUEST);
	ini_set('magic_quotes_gpc', 0);
}

set_magic_quotes_runtime(0);

// register_globals off? Well I like it...
extract($_REQUEST);

// security check
if (!empty($id))
{
    validate_input($id);
}

// create smarty object
$smarty = new Smarty;
$smarty->compile_dir	= './cache/smarty';	            // path to compiled templates
$smarty->cache_dir		= './cache/smarty';	            // path to cached html
$smarty->plugins_dir    = array('custom', 'plugins');   // add custom plugins folder
$smarty->use_sub_dirs	= 0;                            // restrict caching to one folder
$smarty->load_filter('output', 'trimwhitespace');       // remove whitespace from output

// load config
load_config();

// check authentification data for multiuser
if (basename($_SERVER['PHP_SELF']) != 'login.php') authcheck();



/* --------------------------------------------------------------------*/
// functions follow

/**
 * Output debug info  						
 *
 * @author  Andreas Goetz   <cpuidle@gmx.de> 
 * @param	var	$var   Variable to dump
 */
function dump($var)
{
    global $argv;
    
    if (is_array($var) || is_object($var))
        $var = print_r($var, 1);    
    $var .= (count($argv) > 0) ? "\n" : "<br/>";

    echo $var;
}

/**
 * Like html_entity_decode() but also supports numeric entities
 *
 * @author www.php.net
 * @param  string   $string  html entity loaded string
 * @return string            html entity free string 
 */
function html_entity_decode_all($string) 
{
    // replace numeric entities
    $string = preg_replace('~&#x([0-9a-f]+);~ei', 'chr(hexdec("\\1"))', $string);
    $string = preg_replace('~&#([0-9]+);~e', 'chr(\\1)', $string);
    
    // replace literal entities
    $trans_tbl = get_html_translation_table(HTML_ENTITIES);
    $trans_tbl = array_flip($trans_tbl);
    
    return strtr($string, $trans_tbl);
}

/**
 * Used to remove magic quotes from the $_GET, $_POST, $_COOKIE and
 * $_SESSION super global arrays. It's automatically called in
 * functions.php
 *
 * @param array &$array Reference to an array
 */
function remove_magic_quotes(&$array)
{
	foreach (array_keys($array) as $key)
	{
		if (is_array($array[$key])) {
			remove_magic_quotes($array[$key]);
		} else {
			$array[$key] = stripslashes($array[$key]);
		}
	}
}

/**
 * magicquote safe formoutputter
 *
 * Strips slashes when magic_quotes_gpc is set.
 *
 * @param  string $name     The input string
 * @return string           The cleaned string
 */
function formvar($name)
{
	if (get_magic_quotes_gpc()) {
		$name = stripslashes($name);
	}
	return htmlspecialchars($name);
	#  return $name;
}

/**
 * Get high resolution time
 *
 * @return integer  current time in microseconds
 */
function getmicrotime()
{
	list($usec, $sec) = explode(' ',microtime());
	return ((float)$usec + (float)$sec);
}

/**
 * SQL function
 *
 * Wrapper for all Database accesses
 *
 * @param  string $sql_string The SQL-Statement to execute
 * @return mixed  either the resultset as an array with hashes or the insertid
 */
function runSQL($sql_string, $verify = true)
{
	global $config;
	
	if ($config['debug'])
    {
		echo "\n\n<!-- $sql_string -->\n\n";
		$timestamp = getmicrotime();
	}

	$link =	mysql_pconnect($config['db_server'], $config['db_user'], $config['db_password']) or
			errorpage('DB Connection Error',
                      "<p>Edit the database settings in <code>".CONFIG_FILE."</code>.</p>
					   <p>Alternatively, consider running the <a href='install.php'>installation script</a>.</p>");

	$res  = mysql_db_query($config['db_database'], $sql_string, $link);

	// mysql_db_query returns either positive result ressource or true/false for an insert/update statement
	if ($res === false)
    {
		if ($verify)
        {
        	// report DB Problem
            errorpage('Database Problem', mysql_error($link)."\n<br />\n".$sql_string);
        }
        else
        {
        	// ignore problem but forward the information
        	$result = false;	
        }
	}
	elseif ($res === true)
	{
        // on insert, return id of created record
		$result = mysql_insert_id($link);
	}
	else
	{
        // return associative result array
        $result = array();

		for ($i=0; $i<mysql_num_rows($res); $i++)
		{
            $result[] = mysql_fetch_assoc($res);
		}
		mysql_free_result($res);
	}
	
	if ($config['debug'])
    {
		$timestamp = getmicrotime() - $timestamp;
		echo "\n\n<!-- time: $timestamp -->\n\n";
	}

	mysql_close($link);
	return $result;
}

/**
 * checks if the page is accessed from within the local net.
 * If not, displays a simple error page and exits
 */
function localnet_or_die()
{
	if (!localnet()) errorpage('Forbidden', 'You are not allowed to access this page');
}

/**
 * Checks if the page is accessed from within the local net.
 *
 * @return  bool  true if localnet
 */
function localnet()
{
	global $config;
	return (preg_match('/'.$config['localnet'].'/', $_SERVER['REMOTE_ADDR']));
}

/**
 * Checks individual cache folder for existance
 * Checks if folder is writable
 * Creates folder if it doesn't exist
 */
function verify_cache_folder($dir)
{
    if (!is_dir($dir))
    {
        if (!@mkdir($dir, 0700)) $error = 'Directory <code>'.$dir.'</code> does not exist.<br/>';
    }
    elseif (!is_writable($dir))
    {
        $error = 'Directory <code>'.$dir.'</code> is not writable.<br/>';
    }
    return $error;
}

/**
 * Checks if the cache directories exist and are writable by the webserver.
 * If they don't exist it tries to create them. If this fails, too a simple
 * error page is displayed.
 * The function checks if the MySQL PHP extensions is loaded, too.
 */
function verify_installation()
{
    global $config;

    // check MySQL extension
    if (!extension_loaded('mysql'))
    {
        errorpage('MySQL extension for PHP not loaded',
                  '<p>The MySQL extension for PHP is not loaded.</p>
                   <p>Please make sure the MySQL module for PHP is installed and enabled
                   in your <code>php.ini</code></p>');
    }

    // collect all directory-specific errors
    $error = '';

    // check cache
    foreach (array('cache' => 0,'cache/smarty' => 0, 'cache/imdb' => 1, 'cache/img' => 1) as $dir => $hierarchical)
	{
        // check top-level folders
        $error .= verify_cache_folder($dir);

        // check hierarchical folders
        if (empty($error) && $config['hierarchical'] && $hierarchical)
        {
            for ($i=0; $i<16; $i++)
            {
                $error .= verify_cache_folder($dir.'/'.dechex($i));
            }
        }
    }

	if (!empty($error))
	{
        errorpage('Cache directories not writable',
                  '<p>The cache directories have to be writable by the webserver!</p>
                   <p>Please fix the following errors:</p>
                   <p>'.$error.'</p>');
    }
}

/**
 * get Thumbnail-URL for an actor
 *
 * @param  string  name of the Actor
 * @return string  the URL to the cached image if exists or a link to img.php
 */
function getActorThumbnail($name, $actorid = 0)
{
	global $config;

    $SQL = 'SELECT name, imgurl, UNIX_TIMESTAMP(NOW()) - UNIX_TIMESTAMP(checked) AS cacheage
              FROM '.TBL_ACTORS;

	// identify actor by unique actor id, of by name
	if ($actorid) {
        $result = runSQL($SQL." WHERE actorid='".addslashes($actorid)."'");
	}
	if (!$actorid || (count($result) == 0)) {
        $result = runSQL($SQL." WHERE name='".addslashes(html_entity_decode($name))."'");
	}

	$imgurl = 'img.php?name='.urlencode($name);
	if ($actorid) $imgurl .= '&amp;actorid='.urlencode($actorid);

	if (!empty($result[0]['name']))
	{
		// really an image?
		if (preg_match('/\.(jpe?g|gif|png)$/i', $result[0]['imgurl'], $matches))
        {            
#			if (checkCacheFile($result[0]['imgurl'], $matches[1], $file)) return($file);
            if (cache_file_exists($result[0]['imgurl'], $cache_file, 'img', $matches[1]))
            {
                return($cache_file);
            }
		}
        elseif ($result[0]['cacheage'] <= $config['thumbAge'])
        {
			// checked only recently
			return(img('nocover.gif'));
		}
	}
	return($imgurl);
}

/**
 * get Thumbnail for a movie
 *
 * @param  string   URL
 * @return string   the URL to the cached image if exists or a link to img.php
 */
function getThumbnail($imgurl)
{
    // really an image?
    if (preg_match('/\.(jpe?g|gif|png)$/i', $imgurl, $matches))
#    if (1==1)
	{
		// local file? - keep it!
		if (!preg_match('/^http/i', $imgurl)) return($imgurl);

        // file in cache?
#		if (checkCacheFile($imgurl, $matches[1], $file))
        if (cache_file_exists($imgurl, $cache_file, 'img', $matches[1]))
        {
            return($cache_file);
		}
        else
        {
            return('img.php?url='.urlencode($imgurl));
		}
	}
    else
    {
		// no image url given -> nopic
		return(img('nocover.gif'));
	}
}

/**
 * Tries to find the given image in template directory then in the default
 * image directory.
 *
 * @param  string  filename of image
 * @return string  path to the image
 */
function img($img)
{
	global $config;

	$result = 'images/'.$img;
	if (file_exists($config['templatedir'].$result)) $result = $config['templatedir'].$result;
	return ($result);
}

/**
 * Load config options from config.inc.php and database and
 * setup sane defaults.
 * Return configuration in global $config array variable
 *
 * @todo    Add security check if install.php is still available
 * @param   boolean force reload of configuration data
 */
function load_config($force_reload = false)
{
	global $config;
	global $lang;
	global $smarty;

	// configuration cached and not outdated?
    if (isset($_SESSION['vdb']['config']) && (!$force_reload) && (!$config['recompile']) &&
        isset($username) && isset($password) &&
        ($_SESSION['vdb']['config_timestamp'] == filemtime(CONFIG_FILE)))
	{
		// load from cache
        $config = $_SESSION['vdb']['config'];
	}
	else
	{
        // check MySQL extension and cache directories
        verify_installation();

		// remember modification time
        $_SESSION['vdb']['config_timestamp'] = filemtime(CONFIG_FILE);

		// get config options from the database
		$SELECT = 'SELECT opt,value
                     FROM '.TBL_CONFIG;
		$result = runSQL($SELECT);

		// put db results into global config array
		foreach ($result as $option)
        {
			$config[$option['opt']] = $option['value'];
		}
		
        // check if database matches the current version
        if ($config['dbversion'] < DB_REQUIRED)
        {
            // run installer
            redirect('install.php?action=upgrade');
/*
            errorpage("Outdated Tablestructure",
                      "<p>Your database does not match the current version of VideoDB! Please use the
                       <code>updatedb.sql</code> script to update your database!</p>
                       <p>On Unix you may use a line like that:<br /><br />
                       <code>mysql -p -f VideoDB < doc/updatedb.sql</code></p>
                       <p>Alternatively, consider running the <a href='install.php?action=upgrade'>installation script</a>
                       to update your database structure.</p>");
*/
        }

		// get user config options from the database
		$user_id = $_COOKIE['VDBuserid'];
		if (!empty($user_id))
		{
			$SELECT = 'SELECT opt, value
                         FROM '.TBL_USERCONFIG.'
			            WHERE user_id = '.$user_id;

			$result = runSQL($SELECT);
			if (count($result))
            {
			    foreach ($result as $option)
                {
			        $config[$option[opt]] = $option[value];
			    }
			}
		}
		
		// set some defaults
		if (empty($config['language'])) $config['language'] = 'en';
		if (empty($config['template'])) $config['template'] = 'modern::compact';
		if (empty($config['filterdefault'])) $config['filterdefault'] = 'unseen';

//		if ($config['IMDBage'] < 1) $config['IMDBage']          = 60*60*24*5;
		if ($config['castcolumns'] < 1) $config['castcolumns']  = 4;
		if ($config['listcolumns'] < 1) $config['listcolumns']  = 1;
		if ($config['thumbAge'] < 1) $config['thumbAge']        = 60*60*24*7*3;
		if ($config['shownew'] < 1) $config['shownew']          = 12;
		
		// prepare som options for later use
		$config['languages']    = @split('::', $config['languageflags']);
		
		// prepare template/style
		$tpl                    = split('::', $config['template']);
		$config['style']        = 'templates/'.$tpl[0].'/'.$tpl[1].'.css';
		$config['templatedir']  = 'templates/'.$tpl[0].'/';

        // smarty cacheid for multiuser mode
        $config['cacheid']      = $tpl[0];

		// check if selected template is valid
		if (!file_exists($config['style']))
		{
			$config['style']       = 'templates/modern/compact.css';
			$config['template']    = 'modern::compact';
			$config['templatedir'] = 'templates/modern/';
		}

        // get installed engines meta information
        if (empty($config['engines']))
        {
            require_once './engines/engines.php';
            $config['engines'] = engineMeta();
        }

        // translate config options of type enginexyz into config[engine]
        foreach ($config['engines'] as $engine => $meta)
        {
            if ($config['engine'.$engine])
            {
                $config['engine'][$engine] = $config['engine'.$engine];
            }
        }
/*
        // added proxy support for $_ENV
        $proxy = $config['proxy_host'];
        if (empty($proxy))
        {
            $env = array_change_key_case($_ENV);
            $proxy = $env['http_proxy'];
        }
        if (!empty($proxy))
        {
            $uri = parse_url($proxy);
            $config['proxy_host'] = ($uri['scheme']) ? $uri['host'] : $uri['path'];
            $config['proxy_port'] = ($uri['port']) ? $uri['port'] : 8080;
        }
*/

		// store loaded configuration
        $_SESSION['vdb']['config'] = $config;
	}
	
	// setup smarty
	$smarty->template_dir = array($config['templatedir'], 'templates/modern');
	
	if ($config['recompile'])
    {
		$smarty->force_compile = 1;
	}
	
    // initialize languages
    $lang = array();

	// load english language as default
	require './language/en.php';

    // override it with local language if nessesary:
    if ($config['language'] != 'en')
    {
        $languages = split('_', $config['language']);
        $file = '';
        foreach ($languages as $language)
        {
            if ($file) $file .= '_';
            $file .= $language;
            @include './language/'.$file.'.php';
        }
    }
}

/**
 * Displays an errorpage and exits
 *
 * @param string $title   The pages headline
 * @param string $body    An additional message
 */
function errorpage($title = 'An error occured', $body = '')
{
    echo '<?xml version="1.0" encoding="en"?>';
?>
    <html xmlns="http:// www.w3.org/1999/xhtml" xml:lang="en" lang="en" dir="ltr">
    <head>
        <title>VideoDB - ERROR</title>
        <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1" />
        <meta name="description" content="VideoDB" />
    </head>
    <body>
        <h1><?=$title?></h1>
        <?=$body?>
    </body>
    </html>
<?
	exit;
}

/**
 * Checks if the logged in user has write permissions for the given video
 *
 * @author Chinamann <chinamann@users.sourceforge.net>
 * @param  integer $id VideoID
 * @return boolean
 */
function check_videopermission($id)
{
    $result = runSQL('SELECT owner_id
                        FROM '.TBL_DATA.'
                       WHERE '.TBL_DATA.'.id = '.$id);
    return check_permission(PERM_WRITE,$result[0]['owner_id']);
}

/**
 * Used to check permissions on a user for a page
 *
 * @author Mike Clark <Mike.Clark@Cinven.com>
 * @author Chinamann <chinamann@users.sourceforge.net>
 * @param  integer $permission Permission to check
 * @param  String  $destUserId UserId to access
 * @return boolean             True if permission exists else false
 */
function check_permission($permission, $destUserId = false)
{
    global $config;
    if ($_SESSION['vdb']['permissions']) $_SESSION['vdb']['permissions'] = array();

    if (!$config['multiuser']) return true;
    if (empty($_COOKIE['VDBuserid']))
    {
        // not logged in
        $permissions = 0;
    }
    else
    {
        if ($destUserId) $destString = (string)('DUID_'.$destUserId);
        else $destString = 'DUID_UNKNOWN';

        // Owner Level Permissions
        if (isset($_SESSION['vdb']['permissions'][$destString]))
        {
            // permission cache
            $permissions = $_SESSION['vdb']['permissions'][$destString];
        }
        else
        {
            if ($config['multiuser'])
            {
                $userid = $_COOKIE['VDBuserid'];
                $SELECT = "SELECT permissions FROM ".TBL_PERMISSIONS." WHERE from_uid='$userid'";
                if ($destUserId) $SELECT .= " AND to_uid=$destUserId";
                $result = runSQL($SELECT);

                $permissions = 0;
                foreach ($result as $perm)
                {
                    $permissions |= $perm['permissions'];
                }
            }
            else
            {
            	$permissions |= PERM_READ;
            }
            	
            $_SESSION['vdb']['permissions'][$destString] = $permissions;
        }

        // User Level Permissions
        if (isset($_SESSION['vdb']['permissions']['USERLEVEL']))
        {
            $permissions |= $_SESSION['vdb']['permissions']['USERLEVEL'];
        }
        else
        {
            $userid = $_COOKIE['VDBuserid'];
            $SELECT = "SELECT permissions FROM ".TBL_USERS." WHERE id=$userid";
            $result = runSQL($SELECT);
            $_SESSION['vdb']['permissions']['USERLEVEL'] = $result[0]['permissions'];
            $permissions |= $_SESSION['vdb']['permissions']['USERLEVEL'];
        }
    }

    // check permissionbits
    return ($permissions & $permission);
}

/**
 * Check permissions on a user for a page and display error message on failure
 *
 * @author  unknown
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   integer $permission  Permission to check (admin,write,writeall)
 * @param   String  $destUserId UserId to access
 */
function permission_or_die($permission, $destUserId = false)
{
    if (!check_permission($permission, $destUserId))
    {
        errorpage("Access denied",'You don\'t have enough permissions to access this
                   page try to <a href="login.php">login</a> first.');
    }
}

/**
 * Checks if the user was authenticated and if the received auth cookie is valid.
 * Function is called for every page except login.php!
 *
 * @param  string $redirect  Redirect to login page if authentication check unsuccessful
 */
function authcheck($redirect = true)
{
    global $config;

    // auth result will always be success for single-user
    $result = true;

    // auth check only in multiuser mode
    if ($config['multiuser'])
    {
        $referer = substr($_SERVER['PHP_SELF'], strrpos($_SERVER['PHP_SELF'],'/')+1) .'?'. $_SERVER['QUERY_STRING'];

        // already logged in?
        $user   = $_COOKIE['VDBusername'];
        $pass   = $_COOKIE['VDBpassword'];
        $userid = $_COOKIE['VDBuserid'];

        // auth cookies present?
        if (!empty($user) && !empty($pass) && !empty($userid))
        {
            // There are some auth cookies - we need to check if they are valid
            if (!(eregi('[a-z]', $user) && eregi('[0-9]', $pass) && eregi('[0-9]', $userid)))
            {
                $result = false;
            }
            else
            {
                // This is the crucial bit, lets just test the cookiecode with SQL again.
                $row = runSQL("SELECT cookiecode FROM ".TBL_USERS." WHERE name='".mysql_escape_string($user)."' AND id=$userid");

                if ($row[0]['cookiecode'] !== $pass && $userid !== $guestid)
                {
                    $result = false;
                }
            }
        }
        else
        {
        	$result = false;
        }

        // HTTP basic authentication (for RSS feed)?
        if (!$result && isset($_SERVER['PHP_AUTH_USER']) && isset($_SERVER['PHP_AUTH_PW']))
        {
            $user   = $_SERVER['PHP_AUTH_USER'];
            $pass   = $_SERVER['PHP_AUTH_PW'];

            // check if basic auth headers are valid
            if (eregi('[a-z]', $user))
            {
                $row = runSQL("SELECT passwd FROM ".TBL_USERS." WHERE name='".mysql_escape_string($user)."'");

                // auth successful if password matches
                $result = (md5($pass) == $row[0]['passwd']);
            }
        }

        // autologin as guest?
        if (!$result && !$config['denyguest'])
        {
            // get script folder for cookie path
            $subdir = substr($_SERVER['PHP_SELF'], 0, strrpos($_SERVER['PHP_SELF'],'/')) . '/';
            $result = runSQL('SELECT name, passwd FROM '.TBL_USERS.' WHERE id = '.$config['guestid']);
            setcookie('VDBusername', $result[0]['name'], null, $subdir);
            setcookie('VDBpassword', $result[0]['passwd'], null, $subdir);
            setcookie('VDBuserid', $config['guestid'], null, $subdir);
            $_COOKIE['VDBusername'] = $result[0]['name'];
            $_COOKIE['VDBpassword'] = $result[0]['passwd'];
            $_COOKIE['VDBuserid']   = $config['guestid'];
            $result = true;
        }

        // goto login page if anything was fishy
        if ($redirect &! $result)
        {
            redirect('login.php?refer='.$referer);
        }
    }

    return $result;
}

/**
 * Function to get the owner name from videodata table
 *
 * @author Mike Clark <Mike.Clark@Cinven.com>
 * @param  integer $id       videodata id
 * @param  boolean $diskid   is the given ID a disk ID instead of videoID?
 * @return string  Returns the owner of the given Video or Disk
 */
function get_owner($id, $diskid = false)
{
    $SELECT = "SELECT ".TBL_USERS.".name AS owner
                 FROM ".TBL_DATA.", ".TBL_USERS."
                WHERE ".TBL_USERS.".id = ".TBL_DATA.".owner_id AND ".TBL_DATA.".";

    if ($diskid)
    {
        $SELECT .= "diskid = '$id'";
    }
    else
    {
        $SELECT .= "id = '$id'";
    }

    $result = runSQL($SELECT);
    return $result[0]['owner'];
}

/**
 * Function to get the owner id from videodata table
 *
 * @author Chinamann <chinamann@users.sourceforge.net>
 * @param  integer $id       videodata id
 * @param  boolean $diskid   is the given ID a disk ID instead of videoID?
 * @return string  Returns the owner of the given Video or Disk
*/
function get_owner_id($id, $diskid = false)
{
    $SELECT = "SELECT owner_id
                 FROM ".TBL_DATA."
                WHERE ";

    if ($diskid)
    {
        $SELECT .= "diskid = '$id'";
    }
    else
    {
        $SELECT .= "id = '$id'";
    }

    $result = runSQL($SELECT);
    return $result[0]['owner_id'];
}

/**
 * Get list of adult genre ids
 *
 * @return  array   Array of adult genre ids
 */
function get_adult_genres()
{
    global $config;

    $adultgenres = array();
    foreach(split('::', $config['adultgenres']) as $ag)
    {
        if (empty($ag)) continue;
        $adultgenres[] = $ag;
    }

    return $adultgenres;
}

/**
 * Checks if a movie is not prohibited because of adults content
 *
 * @param   integer $id   video id
 * @return  boolean       Returns true if access is granted
 */
function adultcheck($id)
{
	global $config;
	
    if (check_permission(PERM_ADULT) || empty($config['adultgenres']))
    {
		// no multiuser or adult genres set or we actually do have the
		// permissions - whatever let's watch some pr0n ;-)
		return true;
	}
	
    $adultgenres = 'genre_id='.join(' OR genre_id=', get_adult_genres());
	$select = 'SELECT video_id
			     FROM '.TBL_VIDEOGENRE.'
			    WHERE video_id = '.$id.'
				  AND ('.$adultgenres.')';
	$result = runSQL($select);	

	return(empty($result[0]['video_id']));
}

/**
 * Checks if the given movie was already seen by the logged in user. If no
 * user is logged in the $seen value is returned
 *
 * Gets username from cookie
 *
 * @param  integer $id    video id
 * @param  boolean $seen  seen
 * @return boolean        True if seen
 */
function get_userseen($id, $seen=0)
{
	if (empty($_COOKIE['VDBusername'])) return $seen;
	$SELECT =  'SELECT video_id
                  FROM '.TBL_USERSEEN.', '.TBL_USERS.'
                 WHERE '.TBL_USERSEEN.'.video_id='.$id." AND
                       ".TBL_USERSEEN.".user_id = ".TBL_USERS.".id AND
                       ".TBL_USERS.".name ='".$_COOKIE['VDBusername']."'";
	$result = runSQL($SELECT);

	$result = (count($result) > 0) ? 1 : 0;
	return($result);
}

/**
 * Sets the status in userseen accordingly to the given seen value
 *
 * Gets username from cookie
 *
 * @param  integer $id    video id
 * @param  boolean $seen  seen
 */
function set_userseen($id,$seen)
{
    if (!empty($_COOKIE['VDBuserid']))
    {
        $user_id= $_COOKIE['VDBuserid'];

		if ($seen) {
            $SEEN = "REPLACE INTO ".TBL_USERSEEN." SET user_id=".$user_id.", video_id='".$id."'";
		} else {
            $SEEN = "DELETE FROM ".TBL_USERSEEN." WHERE user_id=".$user_id." AND video_id='".$id."'";
		}
		runSQL($SEEN);
	}

/*
	// future code when userseen contains more user-specific data
	
	// record already exists?
	$SELECT = "SELECT seen, user_id FROM video_user, users ".
			"WHERE video_user.video_id=".$id." AND video_user.user_id = user.id AND users.user='".$_COOKIE['VDBusername']."'";
	$result = runSQL($SELECT);

	if (empty($result[0]['id'])) {
		$SELECT = "SELECT id FROM users ".
				  "WHERE user='".$_COOKIE['VDBusername']."'";
		$result = runSQL($SELECT);

		$SQL	= "INSERT INTO video_user SET user_id='".$result[0]['id']."', id='".$id."', seen='".$seen."'";
	}
	else {
		$SQL	= "UPDATE video_user SET seen='".$seen."' ".
				  "WHERE user_id='".$result[0]['id']."', id='".$id."', ";
	}
	runSQL($SQL);
*/
}

/**
 * Verify variable is valid according to validation function
 *
 * @author Andreas Goetz <cpuidle@gmx.de>
 * @param  string   $var                variable to validate (e.g. $id)
 * @param  string   $validation_func    validation function name (e.g. is_numeric)
 */
function validate_input(&$var, $validation_func = 'is_numeric')
{
    if (function_exists($validation_func))
    {
        if (!$validation_func($var))
        {
            errorpage('Forbidden', 'You are not allowed to access this page.');
        }
    }
}

/**
 * Display template with Smarty
 * If Smarty caching is enabled and cache id present, then cache will be used
 *
 * @author Andreas Goetz <cpuidle@gmx.de>
 * @param   string  $template   Template file name for display
 * @parem   string  $id         Cache id
 */
function smarty_display($template, $id = null)
{
    global $smarty, $config;

    // config[cacheid] is set to the template name
#    if ($id && $config['smarty_caching']) $smarty->caching = 1;
    $smarty->display($template, $id, $config['cacheid']);    
#    $smarty->caching = 0;    
}

/**
 * Return UserId to a given UserName
 *
 * @author Chinamann <chinamann@users.sourceforge.net>
 * @param   string  $userName user name
 * @result  integer user id
 */
function getUserId($userName)
{
    $SELECT = "SELECT id
                 FROM ".TBL_USERS."
                WHERE name='".$userName."'";
    $result = runSQL($SELECT);
    return $result[0]['id'];
}

/**
 * Return UserName to a given UserId
 *
 * @author Chinamann <chinamann@users.sourceforge.net>
 * @param  integer   $userId user id
 * @param  string            user name
 */
function getUserName($userId)
{
    $SELECT = "SELECT name
                 FROM ".TBL_USERS."
                WHERE id=".$userId;
    $result = runSQL($SELECT);
    return $result[0]['name'];
}

/**
 * Redirect to new location
 *
 * @author  Andreas Goetz   <cpuidle@gmx.de>
 * @param   string  $dest   Redirect destination
 * @todo    Read somewhere that according to RFC redirects need to specify full URI
 */
function redirect($dest)
{
    header('Location: '.$dest);
    exit();
}

?>