<?php
/**
 * Output functions
 *
 * Functions for HTML output generation (Not templates!)
 *
 * @todo    Check if this can be moved to smarty plugins
 *
 * @package Core
 * @author  Andreas Gohr    <a.gohr@web.de>
 * @version $Id: output.php,v 1.16 2005/09/28 11:21:09 andig2 Exp $
 */

require_once './core/functions.php';

/**
 * Convert SQL result array into associative key=>value array
 *
 * @param   $key    key index name
 * @param   $value  value index name
 * @param   $result SQL result array
 * @return  array   resulting associative array
 */
function associateResultSet($key, $value, &$result)
{
    $ary = array();
    foreach ($result as $row)
    {
        $ary[$row[$key]] = $row[$value];
    }
    return $ary;
}

/**
 * Display genre checkboxes
 *
 * @param  array $selected  selected genre IDs
 * @return                  string HTML for genre checkboxes
 */
function out_genres($selected)
{
    global $config;

	$SELECT = 'SELECT id, name
                 FROM '.TBL_GENRES.'
             ORDER BY name';
	$result = runSQL($SELECT);

	$out = '<table class="genreselect"><tr>';

    // get list of adult genres
    $adultgenres = array();
    if ($config['multiuser'] &! check_permission(PERM_ADULT))
    {
        $adultgenres = get_adult_genres();
    }

	$row = 0;
	foreach ($result as $res)
	{
        // don't show adult genres if no permissions
        if (in_array($res['id'], $adultgenres)) continue;

		$out .= '<td nowrap="nowrap">';
		$out .= '<input type="checkbox" name="genres[]" id="genreid'.$res['id'].'" value="'.$res['id'].'"';
		if (@in_array ($res['id'], $selected)) {
			$out .= ' checked="checked"';
		}
		$out .= '/>';
		$out .= '<label for="genreid'.$res['id'].'">';
		$out .= $res['name'];
		$out .= '</label> ';
		$out .= '</td>';
        if ((++$row % 5) == 0)
        {
			$out .= '</tr><tr>';
		}
	}
	$out .= '</tr></table>';
	return $out;
}

/**
 * Display selectbox with available Mediatypes
 *
 * @todo is this still used? can it be replaced by template code?
 * @author <rob@robvonk.com>
 * @return string   HTML of selectbox
 */
function out_mediatypes()
{
	global $config;

    // select mediatypes
	$SELECT = 'SELECT id, name
                 FROM '.TBL_MEDIATYPES.'
			 ORDER BY name';
	$result = runSQL($SELECT);

    // build associative array
    $mediatypes = array('0' => '') + associateResultSet('id', 'name', $result);

    return $mediatypes;
}

/**
 * All available language flags for config screen
 *
 * @param  array  $flags   selected flags
 * @return string          HTML of Languageflags
 */
function out_languageflags($flags)
{
	global $config;

	$out	= '';
	$count	= 1;
	
	if (($dh = @opendir('./'.$config['templatedir'].'images/flags')) || ($dh = opendir('./images/flags')))
	{
		while (($file = readdir($dh)) !== false)
		{
			if (preg_match("/(.*)\.gif$/", $file, $matches))
			{
				$CHECK= (in_array($matches[1], $flags)) ? 'checked="checked"' : '';
				$out .= '<input type="checkbox" name="languages[]" '.$CHECK.' id="flag_'.$matches[1].'" value="'.$matches[1].'" />';
				$out .= '<label for="flag_'.$matches[1].'">';
				$out .= '<img src="'.img('flags/'.$matches[1].'.gif').'" width="30" height="15" alt="'.ucwords($matches[1]).'" title="'.ucwords($matches[1]).'" />';
				$out .= '</label> ';
				if ($count++%4 == 0) $out.='<br />';
			}
		}
		closedir($dh);
	}
	return $out;
}

/**
 * List of owners names/ids for edit/index/search templates
 *
 * @author  <cpuidle@gmx.de>
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   string  $prefix         Predefined additional Array entrys
 * @param   string  $permissions    Honor permissions for selectbox
 * @return  string                  Array with keys=ownernames and values=ownerids
 */
function out_owners($prefix = null, $permissions = false,$keyIsId = false)
{
    global $config;

    if ($config['denyguest'])
    {
        $WHERES = " AND B.id != ".$config['guestid'];
    }
    else
    {
        $WHERES = "";
    }

    // select user ids
    if ($permissions)
    {
        global $_COOKIE;

        $SELECT = 'SELECT permissions FROM '.TBL_USERS.
                  " WHERE id = '".$_COOKIE['VDBuserid']."';";
        $result = runSQL($SELECT);

        if (($result[0]['permissions'] & $permissions) == $permissions)
        {
            $SELECT = 'SELECT B.id, B.name
                         FROM '.TBL_USERS.' B
                        WHERE 1=1 '.$WHERES.'
                        ORDER BY B.name';
            $result = runSQL($SELECT);
        }
        else
        {
            $SELECT = 'SELECT DISTINCT(B.name) AS name, B.id
                         FROM '.TBL_PERMISSIONS.' A, '.TBL_USERS." B
                        WHERE A.to_uid = B.id
                          AND A.from_uid = '".$_COOKIE['VDBuserid']."'
                          AND (A.permissions & ".$permissions.') = '.$permissions.$WHERES.'
                        ORDER BY name';
            $result = runSQL($SELECT);
        }
    }
    else
    {
        $SELECT = 'SELECT B.id, B.name
                     FROM '.TBL_USERS.' B
                    WHERE 1=1 '.$WHERES.'
                    ORDER BY B.name';
        $result = runSQL($SELECT);
    }

    if ($keyIsId) $key = 'id';
    else $key = 'name';

    // build associative array
    $owners = array_unique(associateResultSet($key,'name', $result));

    // return directly or Zend Optimizer 2,5,5 will do ugly things
    if ($prefix) return($prefix + $owners);

    return $owners;
}

/**
 * Converts keys of an array to MySQL compatible list
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   Array   $owners        Array from out_owners
 * @param   boolean $isString      return list of strings?
 * @return  string                 MySQL compatible list
 */
function owners2list($owners, $isString = true)
{
    if ($isString) return "'".join("', '", array_keys($owners))."'";
    else return join(", ", array_keys($owners));
}

?>