<?php
/**
 * XML Import/ Export functions
 *
 * Lets you browse through your movie collection
 * Requires PHP 5 for import
 *
 * @package Core
 * @author  Andreas Gtz    <cpuidle@gmx.de>
 * @version $Id: xml.php,v 1.23 2005/12/18 18:16:06 andig2 Exp $
 */

require_once './core/functions.php';
require_once './core/genres.php';
require_once './core/xml_functions.php';

/**
 * Import XML data
 *
 * @param   string  $xmldata    XML input string
 */
function xmlimport($xmldata, &$error)
{
    global $xmloptions;

    // amazonxml requires php5 xpath functions
    if (version_compare(phpversion(), '5.0') < 0)
    {
        errorpage('PHP version mismatch', 
                  'At least PHP version 5.0.0 is required to run the XML module, please check the documentation!');
    }
    
    // check php version and register stream wrapper
    require_once './core/VariableStream.class.php';

    // create DOM document
    $xml = new domDocument();
    $xml->preserveWhiteSpace = false;

    // parse xml
    if (!$xml->load('var://xmldata'))
    {
        $error = 'Error parsing input XML, import cancelled.';
        return false;
    }
    $xpath = new domXPath($xml);

    $items = $xpath->query('item');

    // loop over items
    for ($i = 0; $i < $items->length; $i++) 
    {
        $video = $items->item($i);
        $nodes = $xpath->query('*', $video);

        $data       = array();
        $genre_ids  = array();

        // loop over item data
        for ($j = 0; $j < $nodes->length; $j++) 
        {
/*        
            // use php4-compatible code below instead
            $key    = $nodes->item($j)->nodeName;
            $value  = $nodes->item($j)->nodeValue;
*/
            $node   = $nodes->item($j);
            $key    = $node->nodeName;
            $value  = $node->nodeValue;

            // handle individual attributes
            switch ($key)
            {
                case 'id':   
                    break;
                    
                case 'owner':
                    // import owner?
                    if ($xmloptions['import_owner'])
                    {
                        // check if owner exists
                        $owners = runSQL('SELECT id FROM '.TBL_USERS.' WHERE name=\''.mysql_escape_string($value).'\'');                        
                        if (count($owners))
                        {
                            $data['owner_id'] = $owners[0]['id'];
                        }
                        else
                        {
                            $error = 'Owner '.$value.' doesn\'t exist!';
                            return false;
                        }    
                    }
                    break;
                    
                case 'custom1':
                case 'custom2':
                case 'custom3':
                case 'custom4':
                    // import custom fields?
                    if ($xmloptions['import_custom'])
                    {
                        $data[$key] = $value;  
                    }
                    break;
                    
                case 'diskid':
                    // import disk ids?
                    if ($xmloptions['import_diskid'])
                    {
                        $data[$key] = $value;  
                    }
                    break;
                    
                case 'genres':
                    $genres = $xpath->query('genre', $node);                    
                    
                    // loop over item data
                    for ($g = 0; $g < $genres->length; $g++) 
                    {
                        $genre  = $genres->item($g);    // php4-compatible notation
                        $value  = $genre->nodeValue;
                        $id     = getGenreId($genre);
                        
                        if (empty($id))
                        {
                            $error = 'Genre '.$value.' doesn\'t exist!';
                            return false;
                        }    
                        
                        $genre_ids[] = $id;
                    }    
                    break;
                    
                default:
                    $data[$key] = $value;
            }
        }

        // data to import?
        if (count($data))
        {
            $keys = join(', ', array_keys($data));
            $values = '';
            foreach(array_values($data) as $value)
            {
                if ($values) $values .= ', ';
                $values .= '\''.mysql_escape_string($value).'\'';
            }    
            
            // import base table data
            $SQL = "INSERT INTO ".TBL_DATA." ($keys) VALUES ($values)";
            $video_id = runSQL($SQL);
            
            if ($video_id === false)
            {
                // shouldn't happen
                $error = 'Error running import SQL query: '.$SQL;
                return false;
            }
            
            // import genres data
            setItemGenres($video_id, $genre_ids);
            
            $imported++;
        }    
    }

    if ($imported == 1)
    {
        // return last item created
        return $video_id;
    }
    else
    {
        // return true if > 1 item imported
        return true;
    }    
}


/**
 * Export XML data
 *
 * @param   string  $where  WHERE clause for SQL statement
 */
function xmlexport($WHERE)
{
    global $config;
    
    // includes genres table in query?
    $TABLE = '';
    if (strstr($WHERE, 'genre_id'))
    {
        $TABLE  = TBL_GENRE.', ';
        $WHERE .= ' AND '.TBL_DATA.'.id = '.TBL_GENRE.'.video_id';
    }
    
    $SQL = 'SELECT '.TBL_DATA.'.*, 
                   '.TBL_USERS.'.name AS owner, 
                   '.TBL_MEDIATYPES.'.name AS mediatype
              FROM '.$TABLE.TBL_DATA.'
         LEFT JOIN '.TBL_USERSEEN.' ON '.TBL_DATA.'.id = '.TBL_USERSEEN.'.video_id AND 
                   '.TBL_USERSEEN.'.user_id = \''.addslashes($_COOKIE['VDBuserid']).'\'              
         LEFT JOIN '.TBL_USERS.' ON '.TBL_DATA.'.owner_id = '.TBL_USERS.'.id
         LEFT JOIN '.TBL_MEDIATYPES.' ON mediatype = '.TBL_MEDIATYPES.'.id
             WHERE '.$WHERE;

    $result = runSQL($SQL);
    
    // do adultcheck
    if (is_array($result))
    {
        $result = array_filter($result, create_function('$video', 'return adultcheck($video["id"]);'));
    }
    
    $xml = '';
    
    // loop over items
    foreach ($result as $item)
    {
        $xml_item = '';
        
        // loop over attributes
        foreach ($item as $key => $value)
        {
            if (!empty($value))
            {
                if ($key != 'owner_id')
                {
                    $tag       = strtolower($key);
                    $xml_item .= createTag($tag, trim(html_entity_decode($value)));
                }    
            }
        }

        // this is a hack for exporting thumbnail URLs
        if ($item['imgurl'] && $config['xml_thumbnails'])
        {
            $thumb = getThumbnail($item['imgurl']);
            if (ereg('cache', $thumb))
                $xml_item .= createTag('thumbnail', trim($thumb));
        }
        
        // genres
        $genres = getItemGenres($item['id'], true);
        if (count($genres))
        {
            $xml_genres = '';
            foreach ($genres as $genre)
            {
                $xml_genres .= createTag('genre', $genre['name']);
            }
            $xml_item .= createContainer('genres', $xml_genres);
        }
        
        $xml .= createContainer('item', $xml_item);
    }

    $xml    = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n".createContainer('catalog', $xml);

//    header('Content-type: text/xml');
    $mime   = (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE')) ? 'application/force-download' : 'application/octet-stream';
    header('Content-type: '.$mime);
    header('Content-length: '.strlen($xml));
    header('Content-disposition: attachment; filename=videoDB.xml');

    echo $xml;
}

/**
 * Update RSS File
 *
 * @author  Mike Clark    <mike.clark@cinven.com>
 */
function rssexport($WHERES)
{
    global $config;
    
	// get the latest items from the DB according to config setting
    $SELECT = 'SELECT id, title, plot, created 
                 FROM '.TBL_DATA.' 
             ORDER BY created DESC LIMIT '.$config['shownew'];
	$result = runSQL($SELECT);

    // script root
    $base = 'http://'.$_SERVER['HTTP_HOST'].dirname($_SERVER['PHP_SELF']);
    
	// setup the RSS Feed
    $rssfeed  = "<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n";
	$rssfeed .= '<rss version="2.0">';
	$rssfeed .= '<channel>';
	$rssfeed .= createTag('title', 'VideoDB');
	$rssfeed .= createTag('link', $base.'/index.php?export=rss');
	$rssfeed .= createTag('description', 'New items posted on VideoDB');
	$rssfeed .= createTag('language', 'en-us');
	$rssfeed .= createTag('lastBuildDate', date("D, j M Y G:i:s T"));
	
	// build the <item></item> section of the Feed
	foreach ($result as $item)
	{
        $xml_item  = createTag('title', $item['title']);
        $xml_item .= createTag('link', $base.'/show.php?id='.$item['id']);
        $xml_item .= createTag('description', $item['plot']);
        $xml_item .= createTag('guid', $base.'/show.php?id='.$item['id']);
        $xml_item .= createTag('pubDate', rss_timestamp($item['created']));
        
        $rssfeed  .= createTag('item', $xml_item, false);
	}
	$rssfeed .= '</channel>';
	$rssfeed .= '</rss>';

    header('Content-type: text/xml');
#   header('Content-length: '.rssfeed($xml));
#   header('Content-disposition: filename=rss.xml');
    echo $rssfeed;
}

?>