<?
/**
 * Edit Page
 *
 * The edit form for adding and editing video data
 *
 * @todo    Check if 0000 instead of empty year is needed
 * @todo    Add error message for unknown genres
 *
 * @package videoDB
 * @author  Andreas Gohr <a.gohr@web.de>
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @version $Id: edit.php,v 2.71 2006/02/12 17:51:51 andig2 Exp $
 */

require_once './core/functions.php';
require_once './core/genres.php';
require_once './core/custom.php';
require_once './core/security.php';

function getDiskId()
{
    global $config;

    /*
     * change this if you have some fancy naming style
     */
	
    // how many digits have to be used for DiskId?
    $digits = ($config['diskid_digits']) ? $config['diskid_digits'] : 4;

    /*
     *  Old way automatic DiskID was generated: result was "highest ID + 1"
     *
    $NEXTUSERID = "SELECT LPAD(TRIM(LEADING '0' FROM MAX(LPAD(TRIM(LEADING '0' FROM diskid), 10, '0'))) + 1, ".
                  $digits.", '0') AS max FROM ".TBL_DATA.' WHERE diskid NOT REGEXP "[^0-9]"';
    $result = runSQL($NEXTUSERID);
    return $result[0]['max'];
    */

    // get all DiskIds ordered from DB
    $SQL = "SELECT LPAD(TRIM(LEADING '0' FROM diskid), 10, '0') AS id FROM ".
           TBL_DATA.' WHERE diskid NOT REGEXP "[^0-9]" ORDER BY id';
           // sql looks strange but fixes problems with users who change their
           // diskid_digits while they have already movies in their DB.
    $results = runSQL($SQL);

    // find first 'free' diskId
    $lastid = 0;
    foreach($results as $result)
    {
        $thisid = preg_replace('/^0+/','',$result['id']);
        if ($lastid + 1 < $thisid) break;
        $lastid = $thisid;
    }

    // return the found id
    return str_pad($lastid + 1, $digits, '0', STR_PAD_LEFT);
}


$articles = array('the ', 'la ', 'a ', 'der ', 'die ', 'das ', 'des ', 'dem ', 'den ',
                  'ein ', 'eine ', 'eines ', 'le ', 'el ', "l'", 'il ', 'les ', 'i ',
                  'o ', 'un ', 'los ', 'de ', 'an ', 'una ', 'las ', 'gli ', 'het ',
                  'lo ', 'os ', 'az ', 'ha-', 'een ', 'det ', 'oi ', 'ang ', 'ta ',
                  'al-', 'uno ', "un'", 'ett ', 'mga ', ' ', ' ', 'els ', ' ', ' ');


// check for localnet
localnet_or_die();

// multiuser permission check
if (empty($id))
    permission_or_die(PERM_WRITE);
else
    permission_or_die(PERM_WRITE, get_owner_id($id));

$noError = true;

// XML import
if ($config['xml'] && ($import == 'xml'))
{
    require_once './core/xml.php';

    // xml file upload
    if (isset($_FILES['xmlfile']) && is_uploaded_file($_FILES['xmlfile']['tmp_name']))
    {
        $file    = $_FILES['xmlfile']['tmp_name'];
        $xmldata = file_get_contents($file);
        unlink($file);
    }

    // uploading XML data directly or loaded from file
    if (!empty($xmldata))
    {
        $error      = '';
        $item_id    = 0;

        if (($xmlitems = xmlimport($xmldata, $error)) !== false)
        {
            // multiple items imported
            if ($xmlitems === true)
            {
                redirect('index.php?filter=new');
            }
            // exactly one movie imported?
            else
            {
                redirect('show.php?id='.$xmlitems);
            }
        }
        $smarty->assign('xmlerror', $error);
    }

    // prepare templates
    tpl_page();

    // display templates
    tpl_display('xmlimport.tpl');
    exit;
}

// list of fields to be read/written from/to html form
$imdb_set_fields    = array('md5','title','subtitle','language','diskid','mediatype','comment','disklabel',
                            'imdbID','year','imgurl','director','actors','runtime','country','plot','filename',
                            'filesize','filedate','audio_codec','video_codec','video_width','video_height','istv',
                            'custom1','custom2','custom3','custom4','seen');

// list of fields to be overwritten by refetchAllInfos-Script
$imdb_overwrite_fields    = array('comment','disklabel','imdbID','year','director','actors','runtime','country','plot',
                            'custom1','custom2','custom3','custom4');

// special fields for SQL statements
$db_null_fields = array('runtime', 'filesize', 'filedate', 'video_width', 'video_height');
$db_zero_fields = array('istv', 'seen');

// legacy
if ($imdb) $lookup = 1;

// get default lookup mode (0=ignore, 1=lookup, 2=overwrite) if not set
if (!isset($lookup)) $lookup = (empty($id)) ? $lookup = $config['lookupdefault_new'] : $config['lookupdefault_edit'];

// preload old data for refresh all mechanism
if ($lookup == 3)
{
    // get a list of movies in DB
    $video = runSQL('SELECT * FROM '.TBL_DATA.' WHERE id = '.$id);

    // get fields (according to list) from db to be saved later
    foreach ($video[0] as $name => $val)
    {
       if (in_array($name, $imdb_set_fields)) $$name = $val;
    }
    $owner_id = $video[0]['owner_id'];

    // Build a list of all fields which are allowed to be overwritten
    $overwrites = array();
    foreach ($imdb_set_fields as $field)
    {
        $tempFieldName = 'update_'.$field;
        if (isset($$tempFieldName) && $$tempFieldName == 1) $overwrites[] = $field;
    }
    $imdb_set_fields = $overwrites;

    $lookup = 2;
}

// lookup imdb
if ($lookup && $imdbID)
{
    require_once './engines/engines.php';

    // get engine from id
    if (empty($engine)) $engine = engineGetEngine($imdbID);

    // get external data
    $imdbdata = engineGetData($imdbID, $engine);

    // lookup cover
    if (empty($imgurl) || ($lookup > 1))
    {
        $imgurl = $imdbdata['coverurl'];
    }

    // lookup genres
    if (count($genres) == 0 || ($lookup > 1))
    {
        $genres = array();
        $gnames = $imdbdata['genres'];
        if (isset($gnames))
        {
            foreach ($gnames as $gname)
            {
                // check if genre is found- otherwise fail silently
                if (is_numeric($genre = getGenreId($gname)))
                {
                    $genres[] = $genre;
                }
            }
        }
    }

    // lookup actors
    if (empty($actors) || ($lookup > 1))
    {
        $actors = $imdbdata['cast'];
    }

    // lookup all other fields
    foreach (array_keys($imdbdata) as $name)
    {
        if (in_array($name, array('coverurl', 'genres', 'cast', 'id'))) continue;

        // use !$$ as empty($$) doesn't seem to work
        if (!$$name || ($lookup > 1))
        {
            $$name = $imdbdata[$name];
        }
    }

    // custom fields
    for ($i=1; $i<=4; $i++)
    {
        $custom = 'custom'.$i;
        $type   = $config[$custom.'type'];
        if (!empty($type) && isset($$type))
        {
            // copy imdb data into corresponding custom field
            $$custom = $$type;
        }
    }
}

// get fields from db if copying
if ($copy && $copyid)
{
    $video = runSQL('SELECT * FROM '.TBL_DATA.' WHERE id = '.$copyid);

    // get fields (according to list) from db to be saved later
    foreach ($video[0] as $name => $val)
    {
        // don't copy diskid
        if ($name == 'diskid')
        {
            if ($config['autoid'])
            {
                $$name = getDiskId();
            }
        }
		else
        {
            if (in_array($name, $imdb_set_fields)) $$name = $val;
        }
    }
}

// explicit setting of owner
if ($config['multiuser'] && empty($owner_id) && empty($id))
{
    if (check_permission(PERM_WRITE, $_COOKIE['VDBuserid']))
    {
        $owner_id = $_COOKIE['VDBuserid'];
    }
    elseif (count($owners = out_owners(false, PERM_WRITE, true)) == 1)
    {
        // check if there is only one owner available
        $owners   = array_keys($owners);
        $owner_id = $owners[0];
    }
    else
    {
        // error no owner specified
        $video = array();
		    		
        // select all fields according to list, plus id
        foreach ($imdb_set_fields as $name)
        {
            $video[0][$name] = $$name;
        }
        $CLIENTERROR = $lang['warn_noOwner'];
        $noError = false;
    }
}

// save data
if ($save)
{
    // use 0000 for empty year- if not set, current year is used
    if (empty($year)) $year = '0000';

    // Fix for Bugreport [1122052] Automatic DiskID generation problem
    if ($config['autoid'] && !empty($diskid) && ($diskid == $autoid))
    {
        // in case DiskID is already used in meanwhile
        // -> update to new DiskId
        $diskid = getDiskId();
    }

    // set owner
    if (!empty($owner_id))
        $SETS = 'owner_id = '.addslashes($owner_id);

    // set default mediatype
    if (empty($mediatype))
        $mediatype = MEDIA_WISHLIST;
        
    // update all fields according to list
    foreach ($imdb_set_fields as $name)
    {
        // sanitize input
        $$name = removeEvilTags($$name);

        // make sure no formatting contained in basic data
        if (in_array($name, array('title', 'subtitle')))
        {
            $$name = trim(strip_tags($$name));

            // string leading articles?
            if ($config['removearticles'])
            {
                foreach ($articles as $article)
                {
                    if (preg_match("/^$article+/i", $$name))
                    {
                        $$name = trim(preg_replace("/(^$article)(.+)/i", "$2, $1", $$name));
                        break;
                    }
                }
            }
        }

        $SET = "$name = '".addslashes($$name)."'";

        if (empty($$name))
        {
            if (in_array($name, $db_null_fields))
                $SET = "$name = NULL";
            elseif (in_array($name, $db_zero_fields))
                $SET = "$name = 0";
        }

        if ($SETS) $SETS .= ', ';
        $SETS .= $SET;
    }

    if ($noError)
    {
        if ($id)
        {
            // update existing record
            // if the disk was on the wishlist and is now available, make sure it appears under 'new'
            if (($oldmediatype == MEDIA_WISHLIST) && ($mediatype != MEDIA_WISHLIST))
            {
                $SETS .= ', created = NOW()';
            }

            $INSERT = 'UPDATE '.TBL_DATA.' SET '.$SETS.' WHERE id = '.$id;
            runSQL($INSERT);
        }
        else
        {
            // insert new record
            $INSERT = 'INSERT INTO '.TBL_DATA.' SET '.$SETS.', created = NOW()';
            $id = runSQL($INSERT);
        }
	
        // save genres
        setItemGenres($id, $genres);

        // set seen for currently logged in user
        set_userseen($id, $seen);

        // uploaded cover?
        if (isset($_FILES['coverupload']) && is_uploaded_file($_FILES['coverupload']['tmp_name']))
        {
            $file = $_FILES['coverupload']['tmp_name'];
            $mime = $_FILES['coverupload']['type'];

            // determine file extension
            if (preg_match('=image/jpe?g=i', $mime))
            {
                $ext = 'jpg';
            }
            elseif (preg_match('=image/(gif|png)=i', $mime, $m))
            {
                $ext = $m[1];
            }
            elseif (preg_match('=application/octet-stream=i', $mime))
            {
                if (preg_match("/\.(jpe?g|gif|png)$/i", $_FILES['coverupload']['name'], $matches))
                {
                    $ext = $matches[1];
                }
            }

            // move to cache and update db
            if (!empty($ext))
            {
                $coverfile = 'cache/img/'.$id.'.'.$ext;
                if (move_uploaded_file($file, $coverfile))
                {
                    $sql = "UPDATE ".TBL_DATA." SET imgurl='$coverfile' WHERE id=$id";
                    runSQL($sql);
                }
            }
	    }

        // clear smarty cache for this item
        $smarty->clear_cache($id);

	    // add another?
	    if ($add_flag)
	    {
	        // remove id to prevent edit mode instead of new
	        $id = '';
	        $smarty->assign('add_flag', $add_flag);
	    }
	    else
	    {
	        // show the saved movie
	        redirect('show.php?id='.$id);
	    }
	}
}


// load existing data
if ($id)
{
	// select all fields according to list, plus id
	foreach ($imdb_set_fields as $name)
    {
		if ($SELECT) $SELECT .= ', ';
		$SELECT .= $name;
	}

    $SELECT = 'SELECT '.TBL_DATA.'.id, '.TBL_DATA.'.owner_id, '.TBL_USERS.'.name AS owner, '.$SELECT.'
                 FROM '.TBL_DATA.'
                 LEFT JOIN '.TBL_USERS.' ON '.TBL_DATA.'.owner_id = '.TBL_USERS.'.id
                WHERE '.TBL_DATA.'.id = '.$id;
	$video = runSQL($SELECT);

	// get seen status for currently logged in user
	$video[0]['seen']	= get_userseen($id,$video[0]['seen']);
	
	// diskid to global scope:
	$diskid = $video[0]['diskid'];
} else {
	$video[0]['language'] = $config['langdefault'];
}


// assign automatic disk id
if (($config['autoid']) && (empty($diskid) || $add_flag))
{
    $video[0]['diskid'] = getDiskId();
	// Fix for Bugreport [1122052] Automatic DiskID generation problem
	$smarty->assign('autoid', $result[0]['max']);
}

if (empty($video[0]['owner_id']) && !empty($owner_id))
{
    $video[0]['owner_id'] = $owner_id;
}

/*print '<pre>';
print_r($video);
print '</pre>';*/

// prepare templates
tpl_page();
tpl_edit($video);

$smarty->assign('lookup_id', $lookup);
$smarty->assign('http_error', $CLIENTERROR);

if ($config['multiuser'])
{
    $smarty->assign('owners', out_owners(array('0' => ''), PERM_WRITE, true));
}

// allow XML import
if ($config['xml'] && empty($id)) $smarty->assign('xmlimport', true);

// display templates
tpl_display('edit.tpl');	

?>
