<?php
/**
 * Allocine Parser
 *
 * Parses data from the Allocine.fr
 *
 * @package Engines
 * @author  Douglas Mayle   <douglas@mayle.org>
 * @author  Andreas Gohr    <a.gohr@web.de>
 * @link    http://www.allocine.fr  Internet Movie Database
 * @version $Id: allocine.php,v 1.3 2005/05/30 11:29:28 andig2 Exp $
 */

$GLOBALS['allocineServer']	= 'http://www.allocine.fr';

/**
 * Get meta information about the engine
 *
 * @todo    Include image search capabilities etc in meta information
 */
function allocineMeta()
{
    return array('name' => 'Allocine (fr)');
}

/**
 * Get Url to search Allocine for a movie
 *
 * @author  Douglas Mayle <douglas@mayle.org>
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function allocineSearchUrl($title)
{
	global $allocineServer;
	return $allocineServer.'/recherche?motcle='.urlencode($title).'&rub=1';
}

/**
 * Get Url to visit Allocine for a specific movie
 *
 * @author  Douglas Mayle <douglas@mayle.org>
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string	$id	The movie's external id
 * @return  string		The visit URL
 */
function allocineContentUrl($id)
{
    	list($engineword, $allocineID) = split(":",$id,2);
	global $allocineServer;
	return $allocineServer.'/film/fichefilm_gen_cfilm='.$allocineID.'.html';
}

/**
 * Search a Movie
 *
 * Searches for a given title on Allocine and returns the found links in
 * an array
 *
 * @author  Douglas Mayle <douglas@mayle.org>
 * @author  Tiago Fonseca <t_r_fonseca@yahoo.co.uk>
 * @author  Charles Morgan <cmorgan34@yahoo.com>
 * @param   string    The search string
 * @return  array     Associative array with id and title
 */
function allocineSearch($title)
{
    global $allocineServer;
    global $CLIENTERROR;

    $resp = httpClient($allocineServer.'/recherche?motcle='.urlencode($title).'&rub=1', 1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // direct match (redirecting to individual title)?
    $single = array();
    if (preg_match('/^'.preg_quote($allocineServer,'/').'\/film\/fichefilm_gen_cfilm=(\d+)\.html', $resp['url'], $single))
    {
        $ary[0]['id']   = 'allocine:'.$single[2];
        $ary[0]['title']= $title;
        return $ary;
    }

    // multiple matches
    preg_match_all('/<A HREF="\/film\/fichefilm_gen_cfilm=(\d+)\.html".*>(.+)<\/A><\/h4>(.*\n.*<h5.*>.*(\(.*\)).*<\/h5>.*\n.*<\/h4>.*\n.*<\/h4>.*\n.*<\/h4>.*\n.*<h4.*">.*(\(\d+\)))<\/h4>/iU', $resp['data'], $data, PREG_SET_ORDER);
    foreach ($data as $row) 
    {
        $info['id']     = 'allocine:'.$row[1];
        $info['title']  = $row[2];
        // add year (helpful in case of multiple matches)
        if (isset($row[5])) $info['title'] .= ' '.$row[5];
        if (isset($row[4])) $info['title'] .= ' '.$row[4];
        $ary[]          = $info;
    }

    return $ary;
}

/**
 * Fetches the data for a given Allocine-ID
 *
 * @author  Douglas Mayle <douglas@mayle.org>
 * @author  Tiago Fonseca <t_r_fonseca@yahoo.co.uk>
 * @param   int   IMDB-ID
 * @return  array Result data
 */
function allocineData($imdbID) 
{
    global $allocineServer;
    global $CLIENTERROR;

    $data= array();	// result
    $ary = array();	// temp

    //Split allocine id into two parts.
    list($engineword, $allocineID) = split(":",$imdbID,2);

    // fetch mainpage
    $resp = httpClient($allocineServer.'/film/fichefilm_gen_cfilm='.$allocineID.'.html', 1);		// added trailing / to avoid redirect
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // Titles
    preg_match('/<TITLE>(.*?)<\/TITLE>/i', $resp['data'], $ary);
    list($t, $s)     = split(" - ",trim($ary[1]),2);
    $data['title']    = trim($t);
    $data['subtitle'] = trim($s);

    // Year
    preg_match('/<A HREF="\/film\/agenda_gen_date=\d+\/\d+\/(\d+).html".*?>/i', $resp['data'], $ary);
    if (!empty($ary[1]))
    {
        $data['year'] = trim($ary[1]);
    }
    else
    {
    	preg_match('/<h4>Ann.{1}e de production : (\d+)<\/h4>/i', $resp['data'], $ary);
    	$data['year']     = trim($ary[1]);
    }

    // Cover URL
    preg_match('/<IMG SRC="([^"]+)?" (border|alt)(.*\n){5,6}.*Date de sortie/i', $resp['data'], $ary);
    $data['coverurl'] = trim($ary[1]);

    // Runtime
    preg_match('/Dur.{1}e : (\d+)h\.{0,1}( (\d+)min\.{0,1}){0,1}<\/h4>/si', $resp['data'], $ary);
    $hours  = preg_replace('/,/', '', trim($ary[1]));
    $minutes  = preg_replace('/,/', '', trim($ary[3]));
    $data['runtime']  = $hours * 60 + $minutes;

    // Director
    preg_match('/R.{1}alis.{1} par\s+?<A.+HREF="\/personne[?\/].+?">(.+?)<\/A>/i', $resp['data'], $ary);
    $data['director'] = trim($ary[1]);

    // Countries
    preg_match_all('/<h4>Film (.+?).<\/h4>.*\n.*<h4>Genre/i', $resp['data'], $ary, PREG_PATTERN_ORDER);
    $originlist  = split(",",trim(join(', ', $ary[1])));
    foreach($originlist as $origin)
    {
        if (preg_match_all('/am.{1}ricain/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Etats-Unis';
            else
                $data['country']  = $data['country'].', Etats-Unis';
        } elseif (preg_match_all('/indien/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Inde';
            else
                $data['country']  = $data['country'].', Inde';
        } elseif (preg_match_all('/allemand/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Allemagne';
            else
                $data['country']  = $data['country'].', Allemagne';
        } elseif (preg_match_all('/britannique/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Royaume-Unis';
            else
                $data['country']  = $data['country'].', Royaume-Unis';
        } elseif (preg_match_all('/fran.{1}ais/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'France';
            else
                $data['country']  = $data['country'].', France';
        } elseif (preg_match_all('/ta(.{1})wanais/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Ta'.$junk[1][0].'wan';
            else
                $data['country']  = $data['country'].', Ta'.$junk[1][0].'wan';
        } elseif (preg_match_all('/n.{1}o-z(.{1})landais/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Nouvelle-Z'.$junk[1][0].'lande';
            else
                $data['country']  = $data['country'].', Nouvelle-Z'.$junk[1][0].'lande';
        } elseif (preg_match_all('/hong\-kongais/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Hong-Kong';
            else
                $data['country']  = $data['country'].', Hong-Kong';
        } elseif (preg_match_all('/mexicain/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Mexique';
            else
                $data['country']  = $data['country'].', Mexique';
        } elseif (preg_match_all('/roumain/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Roumanie';
            else
                $data['country']  = $data['country'].', Roumanie';
        } elseif (preg_match_all('/belge/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Belgique';
            else
                $data['country']  = $data['country'].', Belgique';
        } elseif (preg_match_all('/luxembourgeois/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Luxembourg';
            else
                $data['country']  = $data['country'].', Luxembourg';
        } elseif (preg_match_all('/irlandais/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Irlande';
            else
                $data['country']  = $data['country'].', Irlande';
        } elseif (preg_match_all('/tha(.{1})landais/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Tha'.$junk[1][0].'lande';
            else
                $data['country']  = $data['country'].', Tha'.$junk[1][0].'lande';
        } elseif (preg_match_all('/danois/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Danemark';
            else
                $data['country']  = $data['country'].', Danemark';
        } elseif (preg_match_all('/tch(.{1})que/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Tch'.$junk[1][0].'coslovaquie';
            else
                $data['country']  = $data['country'].', Tch'.$junk[1][0].'coslovaquie';
        } elseif (preg_match_all('/chinois/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Chine';
            else
                $data['country']  = $data['country'].', Chine';
        } elseif (preg_match_all('/japonais/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Japon';
            else
                $data['country']  = $data['country'].', Japon';
        } elseif (preg_match_all('/espagnol/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Espagne';
            else
                $data['country']  = $data['country'].', Espagne';
        } elseif (preg_match_all('/canadien/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Canada';
            else
                $data['country']  = $data['country'].', Canada';
        } elseif (preg_match_all('/italien/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Italie';
            else
                $data['country']  = $data['country'].', Italie';
        } elseif (preg_match_all('/australien/i', $origin, $junk, PREG_PATTERN_ORDER) > 0) {
            if (trim($data['country']) == '')
                $data['country']  = 'Australie';
            else
                $data['country']  = $data['country'].', Australie';
        } else {
            if (trim($data['country']) == '')
                $data['country']  = $origin;
            else
                $data['country']  = $data['country'].', '.$origin;
        }
    }

    // Plot
    preg_match('/<b>Synopsis<\/b>[^\n]+\n[^\n]*\n[^\n]*\n[^\n]*\n[^\n]*<h4>(.+)<\/h4>/Us', $resp['data'], $ary);
    if (!empty($ary[1])) $data['plot'] = trim($ary[1]);
    // And cleanup
    $data['plot'] = preg_replace('/[\n\r]/',' ', $data['plot']);
    $data['plot'] = preg_replace('/  /',' ', $data['plot']);
    $data['plot'] = trim($data['plot']);

    // Genres (as Array)
    if (preg_match_all('/<h4>Genre : (.+)<\/h4>/i', $resp['data'], $ary, PREG_PATTERN_ORDER) > 0)
    {
        $genrelist     = split(",",trim(join(', ', $ary[1])));
        $genreindex = 0;
        foreach($genrelist as $genre)
        {
            if (preg_match_all('/Com.{1}die/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Comedy';
            } elseif (preg_match_all('/Dessin anim.{1}/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Animation';
            } elseif (preg_match_all('/Animation/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Animation';
            } elseif (preg_match_all('/Espionnage/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                // There is no category for spy.
                $data['genres'][]  = 'Action';
                $data['genres'][]  = 'Thriller';
                $data['genres'][]  = 'Adventure';
            } elseif (preg_match_all('/Karat.{1}/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                // There is no category for Karate.
                $data['genres'][]  = 'Action';
            } elseif (preg_match_all('/Divers/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                // There is no category for Diverse.
            } elseif (preg_match_all('/Biographie/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                // There is no category for history.
                $data['genres'][]  = 'Documentary';
            } elseif (preg_match_all('/Historique/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                // There is no category for history.
                $data['genres'][]  = 'War';
            } elseif (preg_match_all('/Guerre/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'War';
            } elseif (preg_match_all('/Documentaire/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Documentary';
            } elseif (preg_match_all('/Science fiction/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Sci-fi';
            } elseif (preg_match_all('/Policier/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Crime';
            } elseif (preg_match_all('/Musique/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Music';
            } elseif (preg_match_all('/horreur/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Horror';
            } elseif (preg_match_all('/Aventure/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Adventure';
            } elseif (preg_match_all('/Fantastique/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Fantasy';
            } elseif (preg_match_all('/Drame/i', $genre, $junk, PREG_PATTERN_ORDER) > 0) {
                $data['genres'][]  = 'Drama';
            } else {
                $data['genres'][]  = trim($genre);
            }
            $genreindex++;
	}
    }

    // If sub-title is blank, we'll try to fill in the original title for foreign films.
    if (empty($data['subtitle']))
    {
        preg_match('/<h4>Titre original : (.*)?<\/h4>/', $resp['data'], $ary);
        if (!empty($ary[1])) $data['subtitle'] = trim($ary[1]);
    }

    // Genres (as Array)
    if (preg_match_all('/<h4>Genre : (.+)<\/h4>/i', $resp['data'], $ary, PREG_PATTERN_ORDER) > 0)

    // fetch credits
    $resp = httpClient($allocineServer.'/film/casting_gen_cfilm='.$allocineID.'.html', 1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";
    $resp['data'] = preg_replace('/.*?Cast<\/B><\/A>(.*?)<\/TABLE>.*/is', '$1', $resp['data']);

    // Cast
    preg_match('/<b>Acteur\(s\)(.*)<b>/isU', $resp['data'], $Section);
    preg_match_all('/<h5>(.*)<\/h5>.*\n.*<h4><a href="\/personne\/fichepersonne_gen_cpersonne=(\d+)\.html(.*)?<h4>(.+)?<\/h4><\/a>/i', $Section[1], $ary,PREG_PATTERN_ORDER);

    $count = 0;
    while (isset($ary[1][$count])) 
    {
	$cast  .= $ary[4][$count]."::".$ary[1][$count]."::allocine:".$ary[2][$count]."\n";
        $count++;
    }
    $data['cast'] = trim($cast);

    return $data;
}

/**
 * Parses Actor-Details
 *
 * Find image and detail URL for actor, not sure if this can be made
 * a one-step process?  Completion waiting on update of actor
 * functionality to support more than one engine.
 *
 * @author  Douglas Mayle <douglas@mayle.org>
 * @author                Andreas Goetz <cpuidle@gmx.de>
 * @param  string  $name  Name of the Actor
 * @return array          array with Actor-URL and Thumbnail
 */
function allocineActor($name, $actorengineid)
{
    global $allocineServer;
    
    // search directly by yid or via name?
    list($engineword, $actorid) = split(":",$actorengineid,2);
    $url = ($actorid) ? 'http://www.allocine.fr/personne/fichepersonne_gen_cpersonne='.urlencode($actorid).'html' : 'http://www.allocine.fr/recherche/?motcle='.urlencode($name).'&rub=2';
    $resp = httpClient($allocineServer.$url, 1);

    $single = array();
    
        $ary[0][0]='http://www.allocine.fr/personne/fichepersonne_gen_cpersonne=83303.html';
        $ary[0][1]='http://a69.g.akamai.net/n/69/10688/v1/img5.allocine.fr/acmedia/medias/nmedia/18/35/37/77/18406136.jpg';
        return $ary;
    // direct match?
    //if (preg_match('/<a name="headshot" href="(.*?)"><img\s.*?src="(.*?)"/s', $resp['data'], $single))
    if (preg_match('/\/personne\/filmographie_gen_cpersonne=(\d+)\.html.+?<img src="(.+)?".*?\n.*?\n.*?\n.*?\n<div><h4><b>.*?(Acteur|Actrice|producteur).*?<\/b><\/h4><\/div>/is', $resp['data'], $single))
    {
        $ary[0][0]='http://www.allocine.fr/personne/fichepersonne_gen_cpersonne='.$single[1].'html';
        $ary[0][1]=$single[2];
        return $ary;
    } 
    
    // multiple exact matches?
    elseif (preg_match('#<b>Exact Matches</b>.+?<a\s+href="(.*?)">#si', $resp['data'], $single)) 
    {
        if (!preg_match('/http/i', $single[1])) $single[1] = $allocineServer.$single[1];
        $resp = httpClient($single[1], true);
        if (preg_match('/<a name="headshot" href="(.*?)"><img\s+.*?src="(.*?)"/s', $resp['data'], $single)) 
        {
            $ary[0][0] = $resp['url'];	// we're already there, even after a redirect
            $ary[0][1] = $single[2];
            return $ary;
        }
    } 
    
    // multiple approximate matches?
    elseif (preg_match('#<b>Approximate Matches</b>.+?<a\s+href="(.*?)">#si', $resp['data'], $single)) 
    {
        if (!preg_match('/http/i', $single[1])) $single[1] = $allocineServer.$single[1];
        $resp = httpClient($single[1], true);
        if (preg_match('/<a name="headshot" href="(.*?)"><img\s+.*?src="(.*?)"/s', $resp['data'], $single)) 
        {
            $ary[0][0] = $resp['url'];	// we're already there, even after a redirect
            $ary[0][1] = $single[2];
            return $ary;
        }
    } 
    
    // cpuidle@gmx.de:  this seems to be outdated in current IMDB version,
    //                  as code is no longer present in allocine html- remove?
    elseif (preg_match('/<LI><A HREF="(.*?)">'.preg_quote($name).'/i', $resp['data'], $single)) 
    {
        if (!preg_match('/http/i', $single[1])) $single[1] = $allocineServer.$single[1];
        $resp = httpClient($single[1], true);
        if (preg_match('/<a name="headshot" href="(.*?)"><img\s+.*?src="(.*?)"/s', $resp['data'], $single)) 
        {
            $ary[0][0] = $resp['url'];	// we're already there, even after a redirect
            $ary[0][1] = $single[2];
            return $ary;
        }
    }
}

?>
