<?php
/**
 * Amazon Parser
 *
 * Parses data from Amazon.com
 *
 * @package Engines
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @link    http://www.amazon.com Amazon
 * @version $Id: amazoncom.php,v 1.3 2005/09/24 13:36:58 andig2 Exp $
 */

$GLOBALS['amazonComServer']	= 'http://www.amazon.com';

/**
 * Get meta information about the engine
 *
 * @todo    Include image search capabilities etc in meta information
 */
function amazoncomMeta()
{
    return array('name' => 'Amazon', 'stable' => 0);
}

/**
 * Get search Url for an Amazon product
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function amazoncomSearchUrl($title)
{
	global $amazonComServer;
	return $amazonComServer;
}

/**
 * Get search Url to visit external site
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   string  $id The movie's external id
 * @return  string      The visit URL
 */
function amazoncomContentUrl($id)
{
    global $amazonComServer;
    return $amazonComServer.'/exec/obidos/ASIN/'.$id.'/'.AMAZON_TOKEN;
}

/**
 * Search a Movie/DVD/Book etc
 *
 * Searches for a given title on Amazon and returns the found links in
 * an array
 *
 * @author  Andreas Goetz (cpuidle@gmx.de)
 * @param   string    The search string
 * @return  array     Associative array with id and title
 */
function amazoncomSearch($title)
{
	global $amazonComServer, $cache;
	global $CLIENTERROR;

	$post = 'size=10'.
			'&url='.urlencode('index=blended').
			'&field-keywords='.urlencode($title);

    $resp = httpClient($amazonComServer.'/exec/obidos/search-handle-form/102-9499445-6725730', $cache, $post);
	if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    if (preg_match_all('/<a href="\/exec\/obidos\/tg\/detail\/-\/(.+?)\/.*?>(.+?)<\/a>/is', $resp['data'], $data, PREG_SET_ORDER))
    {
		foreach ($data as $row)
        {
			if (ereg('<img', $row[2])) continue;
			$info['id']			= trim($row[1]);
#			$info['showid']		= $row[2];
#			$info['episodeid']	= $row[3];
            $info['title']      = trim(strip_tags($row[2]));
#			$info['subtitle']	= $row[4];
			$ary[]				= $info;
		}
	}
#die(nl2br(print_r($data,1)));

	return $ary;	
}

/**
 * Fetches the data for a given Amazon ID (equals ISBN)
 *
 * @author  Andreas Goetz <cpuidle@gmx.de>
 * @param   int   IMDB-ID
 * @return  array Result data
 */
function amazoncomData($imdbID)
{
	global $amazonComServer, $cache;
	global $CLIENTERROR;

	$data = array();  // result

	// fetch mainpage
	$resp = httpClient($amazonComServer.'/exec/obidos/ASIN/'.$imdbID, 1);
	if (!$resp[success])
    {
		$CLIENTERROR .= $resp[error]."\n";
		return $data;
	}
	# print htmlspecialchars($resp['data']);

    // Title
    if (preg_match("/<b class=\"sans\">(.+?)<\/b>/", $resp['data'], $ary))
    {
        $data['title'] = $ary[1];
    }

	// Cover URL (verify if large image exists)
	if (preg_match("/\"(http:\/\/images(-eu)?.amazon.com\/images\/P\/$imdbID\.0\d\.\w+?.jpg)\"/", $resp['data'], $ary))
    {
		$data['coverurl'] = $ary[1];
	}

    // Director
    if (preg_match("/<a.*?search-handle-url.*?field-author.*?>(.*?)<\/a>/is", $resp['data'], $ary))
    {
        $data['director'] = trim($ary[1]);
    }

	// Language
	if (preg_match("/<b>Sprache:<\/b>(.*?)<br>/is", $resp['data'], $ary))
    {
        $data['language'] = trim(strtolower($ary[1]));
    }

    // Year
    # <b>Erscheinungsdatum:</b> 1. Januar 1994<br>
    if (preg_match("/<b>(DVD )?Release Date:<\/b>.*?(\d{4})(\n|<br>|<li>)/is", $resp['data'], $ary))
    {
        $data['year'] = trim($ary[2]);
    }

    // Runtime
    # <b>Run Time:</b> 575<br>
    if (preg_match("/<b>Run Time:<\/b>\s*(\d{2,})(\n|<br>|<li>)/is", $resp['data'], $ary))
    {
        $data['runtime'] = trim($ary[1]);
    }

	// Plot
    #<a name="amzn-reviews"><b class=h1>Editorial Reviews</b><br></a>
    #<a name="b0009nz2vu7480"> <span class="small"><b><i>Amazon.com</i></b></span><br> <span class="serif">The classic to clunker ratio is still extraordinarily high, though <I>The Simpsons</I>' sixth season could give some devoted viewers pause. The show that takes cheeky delight in mooning television convention gives us "Another Simpsons Clip Show" and its first season-ending cliffhanger, "Who Shot Mr. Burns?" And, as does Bart in "A Star Is Burns," we should all feel a little dirty at the "cheap cartoon crossover" appearance of Jay Sherman (Jon Lovitz), designed to give a boost to the ill-fated animated series <a href="/exec/obidos/ASIN/B00008EY6Q/102-9499445-6725730"><I>The Critic</I></a>. But this is just beard-stroking tongue-clucking regarding a season that delivered episodes that rank in the hallowed <I>The Simpsons</I> pantheon, among them, "Homer Badman," in which lust for a gummy Venus de Milo, peeled from the behind of an unwitting babysitter, makes Homer the object of feminist protest and tabloid TV fodder, and "Homer the Great," in which Homer is discovered to be the Chosen One to lead the secret society, "The Stonecutters" ("Who holds back the electric car/Who made Steve Guttenberg a star?/We do!"). Several episodes take their inspiration from classic films and books: Hitchcock's <I>Rear Window</I> ("Bart of Darkness"); Michael Crichton's <I>Westworld</I> and <I>Jurassic Park</I> ("Itchy and Scratchy Land"); and Stephen King and Ray Bradbury ("Treehouse of Horror V").<p> This season's roster of guest voices is also especially impressive, including Winona Ryder as "Lisa's Rival," Meryl Streep as Rev. Lovejoy's bad-seed daughter ("She's like a Milk Dud," a smitten Bart laments. "Sweet on the outside, poison on the inside"), the late Anne Bancroft in "Fear of Flying"; Patrick Stewart in "Homer the Great"; Mel Brooks and Susan Sarandon in "Homer vs. Patty and Selma," and Mandy Patinkin as Lisa's future fianc&#233;e in the surprisingly moving "Lisa's Wedding." There has, of late, been a feud a-brewin' between fans of <I>The Simpsons</I> and <a href="/exec/obidos/tg/browse/-/12889901/102-9499445-6725730"><I>Family Guy</I></a>. Which show is funnier? Has <I>The Simpsons</I> lost it? Is <I>Family Guy</I> a <I>Simpsons</I>-wannabe? Hey; Can't we all just laugh along? Best to just marvel at another exemplary <I>Simpsons</I> season that, to quote Homer in "Lisa's Rival," delivers it all: "The terrifying lows, the dizzying highs, the creamy middles." <I>--Donald Liebenson</I></span> </a><br><br>
    #<a name="b0009nz2vu5050"> <span class="small"><b><i>Description</i></b></span><br> <span class="serif">"THE SIMPSONS" THE COMPLETE SIXTH SEASON DVD COLLECTION arrives on August 16, 2005 from Fox Home Entertainment. Showcasing all 25 episodes from the acclaimed sixth season, including Part One of the season cliffhanger "Who Shot Mr. Burns?,"</span> </a><br><br>
    if (preg_match('/>Editorial Reviews<\/b>.*?<span class="serif">(.*?)<\/span>/is', $resp['data'], $ary))
    {
        $data['plot'] = trim($ary[1]);
    }

	// Rating
    if (preg_match('/<b>Average Customer Review:<\/b>.*?alt="(\d(\.| out of )\d).*?"/is', $resp['data'], $ary))
    {
	    $data['rating'] = 2 * trim($ary[1]);
    }

	// Genres (as Array)
	if (preg_match('/<b>Kategorie\(n\):<\/b>.*?>(.*?)<br>/is', $resp['data'], $ary))
    {
        $gens = preg_split("/&|,/", $ary[1]);
        #print_r($gens);
        foreach($gens as $genre)
        {
            $genre = trim($genre);
            $genre = strip_tags($genre);
            if (!$genre) continue;
            $data['genres'][] = $genre;
        }
        #print_r($data['genres']);
    }

    // Cast
    if (preg_match("/<a.*?search-handle-url.*?field-actor.*?>(.*?)<\/a>/is", $resp['data'], $ary))
    {
        $data['cast'] = trim($ary[1]);

        if (preg_match("/<b>Starring.+?,&nbsp;<a href=(.+?)>See more<\/a>/i", $resp['data'], $ary))
        {
            $resp = httpClient($amazonComServer.$ary[1], 1);
            if ($resp[success])
            {
                #<B><a href="/exec/obidos/search-handle-url/102-9499445-6725730?index=dvd&field-keywords=Dan Castellaneta ">Dan Castellaneta </a></B>
                #<i>... Homer Simpson/Grampa/Barney Gumble/Krusty the Klown/Groundskeeper Willie/Mayor Quimby/Hans Moleman/Sideshow Mel/Others </i>
                #<br>
                if (preg_match_all('/<B><a href="\/exec\/obidos\/search-handle-url.+?>(.+?)<\/a><\/B>\s*<i>(.+?)<\/i>\s*<br>/is', $resp['data'], $ary, PREG_SET_ORDER))
                {
#                    print_r($ary);
                    $cast = array();

                    foreach ($ary as $row)
                    {
                        $characters = preg_replace("/^\s*\.+\s*/", "", trim($row[2]));
                        $cast[] = trim($row[1]).'::'.$characters;
                    }
                    $data['cast'] = join("\n", $cast);
                }
            }
        }
    }

#print_r($data);

	return $data;
}

?>