<?php
/**
 * DVDB.de Parser
 *
 * Parse data from a german DVD Database
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @package Engines
 * @link    http://www.dvdb.de
 */

$GLOBALS['dvdbServer']   = 'http://www.dvdb.de';
$GLOBALS['dvdbIdPrefix'] = 'dvdb:';

/**
 * Try to Login to DVDB to get FSK18 access
 * 
 * @author  Chinamann <chinamann@users.sourceforge.net>
 */
function dvdbLogin()
{
    global $dvdbServer;
    global $config;

    if ($config['dvdb_loggedin'] != true && !empty($config['dvdb_user']) && !empty($config['dvdb_password']))
    {
        // save session cookies
        $resp = httpClient($dvdbServer."/index.cfm", 0, '', 15, $config['dvdb_cookies']);
        $config['dvdb_cookies'] = get_cookies_from_response($resp, $config['dvdb_cookies']);

        // collect post data
        $post = 'action=logon&content=sites%2Fdvdb%2Fhome_loggedin.cfm&nickname='.
                urlencode($config['dvdb_user']).'&password='.
                urlencode($config['dvdb_password']).'&stayloggedin=1';

        // Login
        $resp = httpClient($dvdbServer."/index.cfm", 0, $post, 15, $config['dvdb_cookies']);

        // save login cookies
        $config['dvdb_cookies'] = get_cookies_from_response($resp, $config['dvdb_cookies']);

        $config['dvdb_loggedin'] = true;
    }

    if ($config['dvdb_simplemode'] != true)
    {
        // switch overview to simple mode
        $resp = httpClient($dvdbServer."/index.cfm?action=setsimplelist", 0, '', 15, $config['dvdb_cookies']);

        // save SimpleMode Cookie
        $config['dvdb_cookies'] = get_cookies_from_response($resp, $config['dvdb_cookies']);

        $config['dvdb_simplemode'] = true;
    }
}                                                                                                             

/**
 * Get meta information about the engine
 * 
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @return  array     Associative array with metadata
 */
function dvdbMeta()
{
    return array('name' => 'DVDB (de)', 'stable' => 1);
}

/**
 * Get Url to search DVDB for a movie
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function dvdbSearchUrl($title)
{
    global $dvdbServer;
    return $dvdbServer.'/index.cfm?action=setmaxrows&content=dvd%2Fquicksearch.cfm&maxrows=9999999&startrow=1&suchbegriff='.urlencode(utf8_encode($title));
}

/**
 * Get Url to visit DVDB for a specific movie
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   string  $id The movie's external id
 * @return  string      The visit URL
 */
function dvdbContentUrl($id)
{
    global $dvdbServer;
    global $dvdbIdPrefix;
    
    // check if login to dvdb is possible -> needed for FSK18 access via trace.php
    dvdbLogin();
    
    $id = preg_replace('/^'.$dvdbIdPrefix.'/', '', $id);
    //return $dvdbServer.'/index.cfm?content=dvd%2Fdetails.cfm&dvd_id='.$id;
    return $dvdbServer.'/index.cfm?action=viewdetails&dvd_id='.$id;
}

/**
 * Search a Movie
 *
 * Searches for a given title on the DVDB and returns the found links in
 * an array
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   string    The search string
 * @return  array     Associative array with id and title
 */
function dvdbSearch($title)
{
    global $dvdbServer;
    global $dvdbIdPrefix;
    global $config;
    global $CLIENTERROR;
    
    // check if login to dvdb is possible -> needed for FSK18 access
    dvdbLogin();

    // get the search result
    $resp = httpClient(dvdbSearchUrl($title), 1,'', 15, $config['dvdb_cookies']);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";
    $resp['data'] = preg_replace('/[\r\n\t]/',' ', $resp['data']);

    // multiple matches
    if (preg_match_all('#<tr.*?<td><a name="([0-9]+)" id="[0-9]+"></a>.*?<td.*?<strong>(.*?)</strong>(.*?)</td>.*?<td>(.*?)</td>.*?<td.*?>(.*?)</td>.*?</tr>#i', $resp['data'], $data, PREG_SET_ORDER))
    {
        foreach ($data as $row)
        {
            if (preg_match('/([0-9]+)/',$row[5],$m))
            {
                $min = ' '.$m[1].'min';
            }
            else $min = '';
            
            $genre = trim($row[4]);
            if ($genre != '') $genre = ' ('.$genre.')';

            if (preg_match('/.*?<br>(.+)/', $row[3], $cy)) 
            {
                if (preg_match('/(.*?)([0-9]+)/', $cy[1], $cz)) 
                {
                    $row[3] = trim($cz[1]);
                    if ($row[3] != '' && $cz[2] != '') $row[3] .= ') (';
                    $row[3] .= $cz[2];
                }
                else $row[3] = trim($cy[1]);
            }
            else $row[3] = "";
            if ($row[3] != '') $row[3] = ' ('.$row[3].')';

            $info['id']     = $dvdbIdPrefix.$row[1];
            $info['title']  = trim($row[2]).$row[3].$genre.$min;
            $info['title']  = preg_replace('/  +/', ' ', $info['title']);
            $ary[]          = $info;
        }
    }
    return $ary;

}

/**
 * Fetches the data for a given DVDB-ID
 *
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @param   int   DVDB-ID
 * @return  array Result data
 */
function dvdbData($dvdbID)
{
    global $dvdbServer;
    global $dvdbIdPrefix;
    global $config;
    global $CLIENTERROR;
    
    $loopLimit = 3; // Maximum retries if http access fails.
    
    $dvdbID = preg_replace('/^'.$dvdbIdPrefix.'/', '', $dvdbID);
    
    // check if login to dvdb is possible -> needed for FSK18 access
    dvdbLogin();

    $data= array(); // result
    $ary = array(); // temp

    // retry until result is OK or loopLimit reached
    for($loop=0;;$loop++) 
    {
    	// fetch mainpage
        $resp = httpClient($dvdbServer.'/index.cfm?action=viewdetails&dvd_id='.$dvdbID, 1, '', 15, $config['dvdb_cookies']);
        if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";
        else 
    	{
            if (preg_match('#<h1>(.*?)</h1>#i', $resp['data'], $ary) && 
                (trim($ary[1]) != 'Request Timeout' || trim($ary[1]) != '')) break;
            if ($loop >= $loopLimit) return $data;
    	} 
    }
	
    $resp['data'] = preg_replace('/[\r\n\t]/',' ', $resp['data']);

    // Titles
    if (preg_match('#<h1>(.*?)</h1>#i', $resp['data'], $ary))
    {
        list($t, $s)      = split(" - ",trim($ary[1]),2);
        $data['title']    = trim($t);
        $data['subtitle'] = trim($s);
    }

    // Country and Year
    if (preg_match('/<td.*?>Land und Jahr<\/td>.*?<td.*?>(.*?)<\/td>/i', $resp['data'], $ary))
    {
        $contries = array();
        foreach (preg_split('/[\/,:]/',$ary[1]) as $country)
        {
            $country = preg_replace('/[0-9]*/','',$country);
            $countries[] = trim($country);
        }
        $data['country']  = trim(join(', ',array_unique($countries)));
        preg_match('/([0-9]{4})/',$ary[1],$ary); // take first year occurrence
        $data['year']     = $ary[1];
    }
    
    // Original Title
    if (preg_match('/<td.*?>Originaltitel<\/td>.*?<td.*?>(.*?)<\/td>/i', $resp['data'], $ary))
    {
        $data['orgtitle'] .= trim($ary[1]);
    }

    // Cover URL
    if (preg_match('/<img.+?src="(dvdpix\/big\/.*?\.jpg)"/i', $resp['data'], $ary))
    {
        $data['coverurl'] =  $dvdbServer.'/'.trim($ary[1]);
    }

    // Runtime
    if (preg_match('#<td.*?>Filml&auml;nge</td>.*?<td.*?>(.*?)</td>#i', $resp['data'], $ary)) {
        if (preg_match('#([0-9]+)#',$ary[1],$ary)) {
            $data['runtime']  = trim($ary[1]);
        }
    }

    // Director
    if (preg_match('/<td.*?><strong>Regie<\/strong><\/td>.*?<td.*?>(.*?)<\/td>/i', $resp['data'], $ary) && 
        preg_match('/<a.*?>&nbsp;(.*?)<\/a>/si', $ary[1], $ary))
    {
        array_shift($ary);
        $data['director'] = trim(join(', ', $ary));
    }

    // Rating
    if (preg_match('/<li>.*?User.Bewertung \(([0-9]{1,2})\/10\)/i', $resp['data'], $ary)) 
    {
        $data['rating'] = $ary[1];
        //$data['comment'] .= "Rating: ".$data['rating']."/10\n";
    } 

    // Languages
    // Languages (as Array)
    $laguages = array(
        'arabisch' => 'arabic',
        'bulgarisch' => 'bulgarian',
        'chinesisch' => 'chinese',
        'tschechisch' => 'czech',
        'dnisch' => 'danish',
        'holndisch' => 'dutch',
        'englisch' => 'english',
        'franzsisch' => 'french',
        'deutsch' => 'german',
        'griechisch' => 'greek',
        'ungarisch' => 'hungarian',
        'islndisch' => 'icelandic',
        'indisch' => 'indian',
        'israelisch' => 'israeli',
        'italienisch' => 'italian',
        'japanisch' => 'japanese',
        'koreanisch' => 'korean',
        'norwegisch' => 'norwegian',
        'polnisch' => 'polish',
        'portugisisch' => 'portuguese',
        'rumnisch' => 'romanian',
        'russisch' => 'russian',
        'serbisch' => 'serbian',
        'spanisch' => 'spanish',
        'schwedisch' => 'swedish',
        'thailndisch' => 'thai',
        'trkisch' => 'turkish',
        'vietnamesisch' => 'vietnamese',
        'kantonesisch' => 'cantonese',
        'katalanisch' => 'catalan',
        'zypriotisch' => 'cypriot',
        'zyprisch' => 'cypriot',
        'esperanto' => 'esperanto',
        'glisch' => 'gaelic',
        'hebrisch' => 'hebrew',
        'hindi' => 'hindi',
        'jdisch' => 'jewish',
        'lateinisch' => 'latin',
        'mandarin' => 'mandarin',
        'serbokroatisch' => 'serbo-croatian',
        'somalisch' => 'somali'
    );
    $lang_list = array();
    if (preg_match('/<td.*?><strong>Tonformate<\/strong><\/td>.*?<td.*?>(.*?)<\/td>/i', $resp['data'], $ary) &&
        preg_match_all('/(\w+):/si', $ary[1], $langs, PREG_PATTERN_ORDER))
    {
        foreach($langs[1] as $language) {
            $language = trim(strtolower($language));
            $language = html_entity_decode(strip_tags($language));
            $language = preg_replace('/\s+$/','',$language);
            if (!$language) continue;
            if (isset($laguages[$language])) $language = $laguages[$language];
            else continue;
            if (!$language) continue;
            $lang_list[] = $language;
        }
        $data['language'] = trim(join(', ', array_unique($lang_list)));
    }

    // Plot
    if(
        preg_match('#<td.*?>Handlung</td>.*?<td.*?>(.*?)</td>#i', $resp['data'], $ary)
    )
    {
        $ary[1] = preg_replace('/<br.*?>/',"\n",$ary[1]);
        $ary[1] = preg_replace('/\s*?$/','',html_entity_decode(strip_tags($ary[1])));
        $data['plot'] = trim($ary[1]);
    }


    // FSK
    if(preg_match('#<td.*>Altersfreigabe</td>.*?<td.*?>(.*?)</td>#', $resp['data'], $ary))
    {
        if (preg_match('#<img.*?alt="(.*?)"#',$ary[1],$fsk)) $fsk = trim(html_entity_decode($fsk[1]));
        else $fsk = trim(html_entity_decode(strip_tags($ary[1])));
        $ary[1] = $fsk;
        
        if (preg_match('#([0-9]+)#',$ary[1],$fsk)) $fsk=$fsk[1];
        elseif (preg_match('#o\.A\.#i',$ary[1])) $fsk="0";
        elseif (preg_match('#o\. A\.#i',$ary[1])) $fsk="0";
        elseif (preg_match('#indiziert#i',$ary[1])) {
            $fsk="18";
            $data['subtitle'] .= ' (indiziert)';
        }
        elseif (preg_match('#Keine Jugendfreigabe#i',$ary[1])) {
            $fsk="18";
        }
        else {
            $ary[1] = preg_replace('/&nbsp;/',' ',$ary[1]);
            $ary[1] = preg_replace('/<br.*?>/',' ',$ary[1]);
            $ary[1] = preg_replace('/\s*?$/','',html_entity_decode(strip_tags($ary[1])));
            $fsk=trim($ary[1]);
        }
    }
    else $fsk = "";
    $data['fsk'] = $fsk;


    // Genres
    $genres = array(
        'Reise' => '',
        'Ratgeber' => '',
        'Revuefilm' => '',
        'Serie' => '',
        'Special' => '',
        'Sport' => 'Sport',
        'TV-Movie' => '',
        'Unterhaltung' => '',
        'Biographie' => 'Biography',
        'Thriller' => 'Thriller',
        'Kriminalfilm' => 'Crime',
        'Science Fiction' => 'Sci-Fi',
        'Kinderfilm' => 'Family',
        'Familie' => 'Family',
        'Dokumentation' => 'Documentary',
        'Action' => 'Action',
        'Actionkomdie' => 'Comedy',
        'Drama' => 'Drama',
        'Abenteuer' => 'Adventure',
        'Historienfilm' => 'History',
        'Monumentalfilm' => '',
        'Komdie' => 'Comedy',
        'Romanze' => 'Romance',
        'Horror' => 'Horror',
        'Splatter' => 'Horror',
        'Western' => 'Western',
        'Erotik' => 'Adult',
        'Klassiker' => '',
        'Eastern' => '',
        'Musikfilm' => 'Musical',
        'Trickfilm' => 'Animation',
        'Anime' => 'Animation',
        'Animation' => 'Animation',
        'Fantasy' => 'Fantasy',
        'Filmoperette' => 'Musical',
        'Horrorkomdie' => 'Comedy',
        'Kriegsfilm' => 'War',
        'Musikdokumentation' => 'Music',
        'Mystery' => 'Mystery',
        'Roadmovie' => 'Thriller',
        'Satire' => 'Comedy',
        'Westernkomdie' => 'Western',
        'Musik' => 'Music',
        'Musik (Oper' => 'Music',
        'Musik (Pop' => 'Music',
        'Musik (Rock' => 'Music',
        'Zeichentrick' => 'Animation'
    );
    preg_match('#<td.*>Genre</td>.*?<a.*?>.*?<img.*?>&nbsp;(.*?)</a>#', $resp['data'], $ary);
    $gens = preg_split('/[\/,]/',$ary[1]);
    foreach($gens as $genre) {
        $genre = trim(html_entity_decode($genre));
        $genre = strip_tags($genre);
        if (!$genre) continue;
        if (isset($genres[$genre])) $data[genres][] = $genres[$genre];
    }

    // Cast
    preg_match('/<td.*?><strong>Darsteller<\/strong><\/td>.*?<td.*?>(.*?)<\/td>/i', $resp['data'], $ary);
    preg_match_all('/<a.*?><img.*?>&nbsp;(.*?)<\/a>/si', $ary[1], $actors, PREG_PATTERN_ORDER);
    $actors = $actors[1];
    $casts = "";
    $role = "";
    $actorid = "";
    foreach($actors as $actor) {
        $actor = trim(strip_tags($actor));
        $actor = preg_replace('/\s+$/','',$actor);
        $actor = preg_replace('/\s+$/','',$actor);
        
        if (preg_match('/:$/',$actor)) continue;
        // $casts .= $actor."::".$role."::".$actorid."\n";
        $casts .= trim($actor)."\n";
        $actorid = "";
    }
    $data['cast'] = trim($casts);

    // EAN-Code
    if (preg_match('#<td.*?>EAN</td>.*?<td.*?>([0-9]+)</td>#i', $resp['data'], $ary))
    {
        $data['barcode'] = $ary[1];
    }

    // Video Aspect Ratio
    if (preg_match('/<td.*?><strong>Bildformate<\/strong><\/td>.*?<td.*?>(.*?)<\/td>/i', $resp['data'], $ary) && preg_match('#:#',$ary[1]))
    {
        $ary[1] = preg_replace('/\s*?$/','',html_entity_decode(strip_tags($ary[1])));
        $ary[1] = preg_replace('/<br>/',', ',$ary[1]);
        $ary[1] = preg_replace('/\s\s+/',' ',$ary[1]);
        // only trust data if ':' is present
        if (preg_match('/:/',$ary[1])) $data['comment'] .= 'Bildformate: '.trim($ary[1])."\n";
    }
    
    // Audio Type
    if (preg_match('/<td.*?><strong>Tonformate<\/strong><\/td>.*?<td.*?>(.*?)<\/td>/i', $resp['data'], $ary) && preg_match('#:#',$ary[1]))
    {
        $ary[1] = preg_replace('/\s*$/','',html_entity_decode($ary[1]));
        $ary[1] = preg_replace('/\s\s+/',' ',$ary[1]);
        $ary = preg_split('/<br>/',$ary[1]);
        if (count($ary)>0) $data['comment'] .= "Tonformate:\n";
        foreach($ary as $audiodesc) 
        {
            if (trim($audiodesc)!='') $data['comment'] .= ' '.trim($audiodesc)."\n";
        }
    }
    
    return $data;
}

/**
 * Get an array of all previous prefixes for the ImdbId
 * 
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @return  array     Associative array with ImdbId prefixes
 */
function dvdbImdbIdPrefixes()
{
	global $dvdbIdPrefix;
    return array($dvdbIdPrefix);
}

?>
