<?php
/**
 * FilmWeb Parser (pl)
 *
 * Parses data from the Polish FilmWeb site
 *
 * @package Engines
 * @author  Marek Domaniuk <domanm@o2.pl>
 * @link    http://www.filmweb.com  Internet Movie Database
 * @version $Id: filmweb.php,v 1.1 2005/06/04 16:20:30 andig2 Exp $
 */

//http://www.filmweb.pl/Find
$GLOBALS['filmwebServer']	= 'http://www.filmweb.pl'; 

/**
 * Get meta information about the engine
 *
 * @todo    Include image search capabilities etc in meta information
 */
function filmwebMeta()
{
    return array('name' => 'FilmWeb (pl)', 'stable' => 1);
}

/**
 * Get Url to search IMDB for a movie
 *
 * @author  Marek Domaniuk <domark@o2.pl>
 * @param   string    The search string
 * @return  string    The search URL (GET)
 */
function filmwebSearchUrl($title)
{
	global $filmwebServer;
	//query=aviator&category=1&submit=szukaj
	return $filmwebServer.'/Find?query='.urlencode($title).'&category=1&submit=szukaj';
	//return $filmwebServer.'/Find';
}

/**
 * Get Url to visit IMDB for a specific movie
 *
 * @author  Marek Domaniuk <domark@o2.pl>
 * @param   string	$id	The movie's external id
 * @return  string		The visit URL
 */
function filmwebContentUrl($id)
{
//http://www.filmweb.pl/Film?id=32882
	global $filmwebServer;
	return $filmwebServer.'/Film?id='.$id.'/';
	//return $filmwebServer.'/Film';
}

/**
 * Search a Movie
 *
 * Searches for a given title on the IMDB and returns the found links in
 * an array
 *
 * @author  Marek Domaniuk <domark@o2.pl>
 * @param   string    The search string
 * @return  array     Associative array with id and title
 */
function filmwebSearch($title)
{
    global $filmwebServer;
    global $CLIENTERROR;

    $resp = httpClient($filmwebServer.'/Find?query='.urlencode($title).'&category=1&submit=szukaj', 1); //query=aviator&category=1&subm
    //$resp = httpClient($filmwebServer.'/Find',1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // direct match (redirecting to individual title)?
    $single = array();
    //if (preg_match('/^'.preg_quote($filmwebServer,'/').'\/[Tt]itle(\?|\/tt)(\d+)\/?/', $resp['url'], $single))
    //AddFilmFavourite?film.id=108407">dodaj do ulubionych</a>
    //if (preg_match('/AddFilmFavourite\?film\.id=(\d+)">dodaj do ulubionych<\/a>/i', $resp['data'], $single))
    if (preg_match('/AddFilmFavourite\?film\.id=(\d+)">dodaj do ulubionych<\/a>/i', $resp['data'], $single))
    {
        $ary[0]['id']   = $single[1];
        $ary[0]['title']= $title;
        return $ary;
    }	

    // multiple matches
//    preg_match_all('/<A HREF="\/title(\?|\/tt)(\d+)\/.*?">(.+?)<\/A>(\s+(\(.+?\)))?/i', $resp['data'], $data, PREG_SET_ORDER);
    //preg_match_all('/title=\'(.*?)\'.+?href=".+?Film\?id=(\d+)">/i', $resp['data'], $data, PREG_SET_ORDER);
    preg_match_all('/title=\'(.*?)\'.+?href="(.+?)">/i', $resp['data'], $data, PREG_SET_ORDER);
    //title='Lotnik / Aviator, The (1985)' href="http://www.filmweb.pl/Film?id=10167">
    foreach ($data as $row) 
    {
        $info['id']     = trim($row[2]);
        $info['title']  = trim($row[1]);
        // add year (helpful in case of multiple matches)
//        if (isset($row[5])) $info['title'] .= ' '.$row[5];
	//select id from http link
        
        if (preg_match('/Film\?id=(\d+)/', $info['id'], $single))
	{
	    $info['id'] = $single[1];
	}
	elseif  ((strpos($info['id'], 'id=')) != true)
	{
        // log response
	    //if ($config['httpclientlog'])
	    //{		        {
//		$log = fopen('httpClient.log', 'a');
//		fwrite($log, "\n\ninformacja: ".$info['id']."\n");
//		fclose($log);
	    //}				    }
						    
	    $subResp = httpClient($info['id'],1);
	    if (!$subResp['success']) $CLIENTERROR .= $subResp['error']."\n";
	    //<a class="n" href="http://www.filmweb.pl/AddFilmFavourite?film.id=108130">dodaj do ulubionych</a>
	    preg_match('/id=(\d+)">dodaj/i', $subResp['data'], $single);
	    $info['id'] = $single[1];
	}
        $ary[] = $info;
    }

    return $ary;
}

/**
 * Fetches the data for a given IMDB-ID
 *
 * @author  Marek Domaniuk <domark@o2.pl>
 * @param   int   IMDB-ID
 * @return  array Result data
 */
function filmwebData($filmwebID) 
{
    global $filmwebServer;
    global $CLIENTERROR;

    $data= array();	// result
    $ary = array();	// temp

    // fetch mainpage
    //$resp = httpClient($filmwebServer.'/', 1);		// added trailing / to avoid redirect
    $resp = httpClient($filmwebServer.'/Film?id='.$filmwebID, 1);		// added trailing / to avoid redirect
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // Titles
    preg_match('/"tyt">\s(.*?)\s+?<span/i', $resp['data'], $ary);
    $data['subtitle'] = trim($ary[1]);
    
    preg_match('/"styt">(.*?) <\/span/i', $resp['data'], $ary);
    $data['title'] = trim($ary[1]);
    if (!$data['title']) 
    {
        $data['title'] = $data['subtitle'];
        $data['subtitle'] = "";
    }
    
    // Year
    preg_match('/"styt">\(([1-2][0-9][0-9][0-9])\)<\/span/i', $resp['data'], $ary);
    $data['year']     = trim($ary[1]);

    // Cover-URL
    //<img  src="http://gfx.filmweb.pl/f/94745/po.6954459.jpg"
    //http://gfx.filmweb.pl/f/520/520.jpg
    preg_match('/<IMG.*?src="http:\/\/(.+?\/(po\.\d+|\d+\/foto|\d+\/\d+)\.(jpe?g|gif))/i', $resp['data'], $ary);
    $url = trim($ary[1]);
    $data['coverurl'] = "http://$url";

    // MPAA Rating
    preg_match('/<A HREF="\/mpaa">MPAA<\/A>.+?<\/B>(.+?)<br>/i', $resp['data'], $ary);
    $data['mpaa']     = trim($ary[1]);

    // UK BBFC Rating
    preg_match('/>UK:(.*?)<\/a>\s+/i', $resp['data'], $ary);
    $data['bbfc'] = trim($ary[1]);
    
    // Runtime
    preg_match('/czas trwania.+?([0-9,]+)\s/si', $resp['data'], $ary);
    $data['runtime']  = preg_replace('/,/', '', trim($ary[1]));

    // Director
    //reyseria				<a class="n" title="Kerry Conran: filmografia [Filmweb.pl]" //href="http://www.filmweb.pl/Kerry,Conran,filmografia,Person,id=138676">Kerry Conran</a>
    preg_match('/reyseria.+?title="(.+?): filmografia.+?"/i', $resp['data'], $ary);
    $data['director'] = trim($ary[1]);

    // Rating
    preg_match('/<b style.+?>(\d{1,2}\,\d{1,3})<\/b>\/10/i', $resp['data'], $ary);
    $data['rating']   = trim($ary[1]);

    // Countries
    preg_match_all('/<A HREF=".+?\/BrowseFilms\?country\.id=\d+">(.+?)<\/A>/i', $resp['data'], $ary, PREG_PATTERN_ORDER);
    $data['country']  = trim(join(', ', $ary[1]));

    // Languages
    //preg_match_all('/<A HREF="\/Sections\/Languages\/.+?\/">(.+?)<\/A>/i', $resp['data'], $ary, PREG_PATTERN_ORDER);
    //$data['language'] = trim(strtolower(join(', ', $ary[1])));

    // Plot
    preg_match('/alt="o filmie".*?<\/div>\s(.*?)<a/is', $resp['data'], $ary);
    if (!empty($ary[1])) $data['plot'] = trim($ary[1]);

    // Genres (as Array)
    $genres = array(
	'Przygodowy' => 'Adventure',
	'Akcja' => 'Action',
	'Komedia' => 'Comedy',
	'Familijny' => 'Family',
	'Muzyka' => 'Music',
	'Western' => 'Western',
	'Dla dorosych' => 'Adult',
	'Krymina' => 'Crime',
	'Fantasy' => 'Fantasy',
	'Musical' => 'Musical',
	'Muzyczny' => 'Musical',
	'Krtkometraowy' => 'Short',
	'Dokumentalny' => 'Documentary',
	'Film-Noir' => 'Film-Noir',
	'Mystery' => 'Mystery',
	'Thriller' => 'Thriller',
	'Dreszczowiec' => 'Thriller',
	'Animacja' => 'Animation',
	'Dramat' => 'Drama',
	'Melodramat' => 'Drama',
	'Dramat historyczny' => 'History',
	'Historyczny' => 'History',
	'Dramat obyczajowy' => 'Drama',
	'Dramat sdowy' => 'Drama',
	'Dramat spoeczny' => 'Drama',
	'Horror' => 'Horror',
	'Romans' => 'Romance',
	'Wojenny' => 'War',
	'Biograficzny' => 'Biographic',
	'Erotyczny' => 'Adult',
	'Komedia kryminalna' => 'Comedy',
	'Komedia obycz.' => 'Comedy',
	'Komedia rom.' => 'Comedy',
	'Komedia' => 'Comedy',
	'Czarna komedia' => 'Comedy',
	'Dla dzieci' => '',
	'Obyczajowy' => '',
	'Bibilijny' => '',
	'Sensacja' => 'Action',
	'Sensacyjny' => 'Action',
	'Fabularyzowany dok.' => 'Documentary',
	'Psychologiczny' => ''
    );
    preg_match_all('/genre\.id=\d+">(.+?)</i', $resp['data'], $ary, PREG_PATTERN_ORDER);
    foreach($ary[1] as $genre)
    {
        $genre = trim($genre);
        if (!($genre)) continue;
        if (isset($genres[$genre])) $genre = $genres[$genre];
        if (!($genre)) continue;
        $data['genres'][] = $genre;
    }

    // Cast
    preg_match_all('/HREF=".+?Person\,id=(\d+)">(.+?)<\/a>.+?\s.+?<td.+?>(.+?)<\/td>/i', $resp['data'], $ary,PREG_PATTERN_ORDER);
    $count = 0;
    while (isset($ary[1][$count])) 
    {
        $actor  = trim(strip_tags($ary[2][$count]));
        $role   = trim(strip_tags($ary[3][$count]));
        $actorid= $ary[1][$count]; //, $m)) ? $m[1] : '';
        $cast  .= "$actor::$role::$actorid\n";
        $count++;
    }
    $data['cast'] = trim($cast);

    // fetch Plot
    $resp = httpClient($filmwebServer.'/FilmDescriptions?id='.$filmwebID, 1);
    if (!$resp['success']) $CLIENTERROR .= $resp['error']."\n";

    // Plot
    preg_match('/<li><div.+?">(.+?)<\/div><\/li>/is', $resp['data'], $ary);
    if (!empty($ary[1])) $data['plot'] = trim($ary[1]);
    $data['plot'] = preg_replace('/[\n\r]/',' ', $data['plot']);
    $data['plot'] = preg_replace('/  /',' ', $data['plot']);
    $data['plot'] = trim($data['plot']);
    
    return $data;
}

/**
 * Parses Actor-Details
 *
 * Find image and detail URL for actor, not sure if this can be made
 * a one-step process?
 *
 * @author                >
 * @param  string  $name  Name of the Actor
 * @return array          array with Actor-URL and Thumbnail
 */
function filmwebActor($name, $actorid)
{
    global $filmwebServer;
    return array();
}

?>
