<?php
/**
 * Browse View
 *
 * Lets you browse through your movie collection
 *
 * @package videoDB
 * @author  Andreas Gohr <a.gohr@web.de>
 * @author  Chinamann <chinamann@users.sourceforge.net>
 * @link	http://videodb.sf.net
 * @version $Id: index.php,v 2.72 2005/10/16 09:52:40 andig2 Exp $
 */

require_once './core/session.php';
require_once './core/functions.php';
require_once './core/output.php';

// set defaults
if (!isset($filter)) 
{
	$filter = $_SESSION['vdb']['filter'];
	$showtv = $_SESSION['vdb']['showtv'];

    if (!isset($filter)) 
    {
        $filter = $config['filterdefault'];
        $showtv = $config['showtv'];
    }
}

$_SESSION['vdb']['filter']  = $filter;
$_SESSION['vdb']['showtv']  = $showtv;

// standard filters
$filter_expr = array(
  'NUM'  => '^["\\\' ]*[^A-Za-z]',
  'ABC'  => '^["\\\' ]*[ABCabc]',
  'DEF'  => '^["\\\' ]*[DEFdef]',
  'GHI'  => '^["\\\' ]*[GHIghi]',
  'JKL'  => '^["\\\' ]*[JKLjkl]',
  'MNO'  => '^["\\\' ]*[MNOmno]',
  'PQRS' => '^["\\\' ]*[PQRSpqrs]',
  'TUV'  => '^["\\\' ]*[TUVtuv]',
  'WXYZ' => '^["\\\' ]*[WXZwxy]'
);

// which seen attribute to use - if logged in use the userseen one
$SEEN = (empty($_COOKIE['VDBusername'])) ? TBL_DATA.'.seen' : '!ISNULL('.TBL_USERSEEN.'.video_id)';

// create SQL according to selected filter
switch ($filter)
{
    case 'all':
                    $WHERES = 'mediatype != '.MEDIA_WISHLIST;
                    if ($config['orderallbydisk']) {
                        $ORDER = 'diskid, title, subtitle';
                    } else {
                        $ORDER = 'title, subtitle';
                    }
                    break;
    case 'unseen':
                    $WHERES = $SEEN.' != 1 AND mediatype != '.MEDIA_WISHLIST;
                    break;
    case 'new':
                    $WHERES = 'mediatype != '.MEDIA_WISHLIST;
                    $ORDER  = 'created DESC, lastupdate DESC ';
                    $LIMIT  = ' LIMIT '.$config['shownew'];
                    break;
    case 'wanted':
                    $WHERES = 'mediatype = '.MEDIA_WISHLIST;
                    break;
    case 'full':
                    $WHERES = '1=1';  // secret filter for exposing all data
                    break;
    default:
                    // make sure filter is valid
                    if (!array_key_exists($filter, $filter_expr)) $filter = 'ABC';
                    // apply filter
                    $WHERES = 'title RLIKE \''.$filter_expr[$filter].'\' AND mediatype != '.MEDIA_WISHLIST;
}

// default order
if (!$ORDER)  $ORDER   = 'title, subtitle';
if (!$showtv) $WHERES .= ' AND istv = 0';

// add owner selection for multiuser mode
if ($config['multiuser']) 
{
    // owner_id stored in session?
    if (empty($owner)) 
    {
        $owner = $_SESSION['vdb']['owner'];
        if (empty($owner)) $owner = $config['owner'];
        if (empty($owner))
        {
        	if (check_permission(PERM_READ, $_COOKIE['VDBuserid'])) 
            {
                $owner = $_COOKIE['VDBusername'];
            }
            elseif (count($owners = out_owners(false, PERM_READ)) == 1) 
            { 
                // check if there is only one owner available
                $owner = $owners[0];
            } 
            else 
            {
                // help! take ALL
                $owner = strtoupper($lang['radio_all']);
            } 
        }
    }
    $_SESSION['vdb']['owner'] = $owner;    

    // build html select box
    $all = strtoupper($lang['radio_all']);

    $smarty->assign('owners', out_owners(array($all => $all), PERM_READ));
    $smarty->assign('owner', $owner);

    // limit visibility
    if ($owner and $owner != $all)
    {
        $WHERES .= " AND ".TBL_USERS.".name = '".addslashes($owner)."'";
    } 
    else 
    {
        $WHERES .= " AND ".TBL_USERS.".name IN (".owners2list(out_owners(null, PERM_READ)).")";	
    }
}

// XML / RSS / PDF export
if ($export && $config[$export])
{
    $func = $export.'export';
    if ($export == 'rss') $export = 'xml';
    require_once './core/'.$export.'.php';

    if (function_exists($func)) $func("$WHERES ORDER BY $ORDER $LIMIT");
    exit();
}

/*
    Calculate pagination
    
    Check to see if user has selected the New items tab.
    This is seperately assigned as a LIMIT so, if this exists, 
    lets just skip page numbers and carry on
*/
if (!$LIMIT && ($config['pageno'] > 0))
{
    // start at first page
    if (empty($pageno)) $pageno = 1;

	// define Max Results Per Page
	$maxresults = $config['pageno'];

	// define the Start Number
	$from = (($pageno * $maxresults) - $maxresults);

	// get total amount of results from DB
	$totalresults = runSQL('SELECT count(*) AS num 
                              FROM '.TBL_DATA.'
                         LEFT JOIN '.TBL_USERSEEN.'
                                ON '.TBL_DATA.'.id = '.TBL_USERSEEN.'.video_id AND '.TBL_USERSEEN.'.user_id = \''.addslashes($_COOKIE['VDBuserid']).'\'
                         LEFT JOIN '.TBL_USERS.'
                                ON '.TBL_DATA.'.owner_id = '.TBL_USERS.'.id
                             WHERE '.$WHERES);

	// calculate Total amount of pages
	$totalpages = ceil($totalresults[0]['num'] / $maxresults);
	
	$LIMIT = ' LIMIT '.$from.', '.$maxresults;

    $smarty->assign('pageno', $pageno);                         // assign current Page Number
	$smarty->assign('maxpageno', $totalpages);		            // set Maximum Pages
	$smarty->assign('totalresults', $totalresults[0]['num']);   // set Total Records Returned
}


// do hard work
$select = 'SELECT '.TBL_DATA.'.id, '.TBL_DATA.'.diskid, 
                  title, subtitle, language, year,
                  director, plot,
                  imgurl, '.TBL_LENT.'.who, '.TBL_USERS.'.name AS owner,
                  md5, comment, disklabel, imdbID, actors, runtime,
                  country, filename, filesize, filedate, audio_codec,
                  video_codec, video_width, video_height, istv,
                  lastupdate, mediatype,
                  custom1, custom2, custom3, custom4, 
                  created, '.$SEEN.' AS seen
             FROM '.TBL_DATA.'
        LEFT JOIN '.TBL_LENT.'
               ON '.TBL_DATA.'.diskid = '.TBL_LENT.'.diskid
        LEFT JOIN '.TBL_USERSEEN.'
               ON '.TBL_DATA.'.id = '.TBL_USERSEEN.'.video_id AND '.TBL_USERSEEN.'.user_id = \''.addslashes($_COOKIE['VDBuserid']).'\'
        LEFT JOIN '.TBL_USERS.'
               ON '.TBL_DATA.'.owner_id = '.TBL_USERS.".id
            WHERE $WHERES
         ORDER BY $ORDER
		   $LIMIT";
#print "<pre>$select</pre>";
$result = runSQL($select);

// store query result in session for prev/next navigation
$_SESSION['vdb']['query_result'] = array_map(create_function('&$row', 'return $row["id"];'), $result);

// prepare
tpl_page('browse');
tpl_list($result);
tpl_filters($filter, $showtv);

// docpart support
if ($docpart)
{
    smarty_display('list.tpl');
    exit;
}

// caching enabled?
if ($config['http_caching'])
{
    require_once('./core/httpcache.php');
    cache_start('index');
}

$smarty->assign('moreless', true);           // show more/less control in list view

// allow XML and RSS export
if ($config['xls']) $smarty->assign('xls', 'index.php?');
if ($config['xml']) $smarty->assign('xml', 'index.php?');
if ($config['pdf']) $smarty->assign('pdf', 'index.php?');
if ($config['rss']) $smarty->assign('rss', 'index.php?');

// display templates
smarty_display('header.tpl');
smarty_display('filters.tpl');
if (!$config['http_caching']) flush();

smarty_display('list.tpl');
smarty_display('footer.tpl');

// caching enabled?
if ($config['httpcaching'])
{
    httpCacheOutput('index', httpCacheCaptureEnd());
}

?>