<?php
/**
 * IMDB lookup popup
 *
 * Popup to search at IMDB
 *
 * @package videoDB
 * @author  Andreas Gohr    <a.gohr@web.de>
 * @author  Andreas Goetz   <cpuidle@gmx.de>
 * @version $Id: lookup.php,v 2.19 2005/03/04 08:37:49 andig2 Exp $
 */
 
require_once './core/functions.php';
require_once './engines/engines.php';

// filter relevant search engines
$engines = $config['engine'];

if ($searchtype == 'image')
{
    // limit to engines capable of image search
    $engines = array_intersect_assoc($engines, array('amazonxml'=>1, 'google'=>1));   // function PHP>=4.3.0
}
else
{
    // Google is not capable of movie searches
    unset($engines['google']);
}

// determine default engine (first in list)
if (empty($engine))
{
    if (!empty($config['enginedefault']))
    {
    	$engine = $config['enginedefault'];
    }
    else
    {
        $engine_list = array_keys($engines);

        if (count($engine_list)) 
        {
            // first valid engine from list
            $engine = $engine_list[0];
        }
        else $engine = 'imdb'; // last resort
    }    
}

// result array
$matches = array();

if (!empty($find))
{
    // decode $find as encodeURI might have added UTF8 characters
    $find = trim($find);    # utf8_decode
    
	switch ($engine)
	{
		case 'amazonxml':
						// amazonxml
						require_once('engines/'.$engine.'.php');
						if (empty($catalog)) $catalog = $searchMode;	// default amazonxml searchmode
						$smarty->assign('searchmode', array('books','music','classical','dvd','vhs'));
						$smarty->assign('modedefault', $catalog);
						if (empty($area)) $area = 'US';					// default amazonxml search area
						$smarty->assign('searcharea', array('US','UK','DE','JP'));
						$smarty->assign('areadefault', $area);

                        $matches    = amazonxmlSearch($find, $catalog, $area);  // catalog and are are only used by amazonxml
                        $searchurl  = amazonxmlSearchUrl($find);
					    break;
                        
						// tvtome, amazon, google
		default:		$matches    = engineSearch($find, $engine);
                        $searchurl  = engineGetSearchUrl($find, $engine);
	}
}

// prepare templates
tpl_language();
tpl_header();

$smarty->assign('engines',  engineGetLookup($find, $searchtype, $engine));
$smarty->assign('engine',   $engine);
$smarty->assign('searchurl',    $searchurl);
$smarty->assign('searchtype',   $searchtype);

$smarty->assign('find', $find);
$smarty->assign('q_find', formvar($find));
$smarty->assign('imdbresults', $matches);
$smarty->assign('http_error', $CLIENTERROR);

// display templates
smarty_display('lookup.tpl');

?>
