<?php
/**
 * Search page
 *
 * Database searches for movies
 *
 * @package Search
 * @author  parts by Justin Pasher <justin@pasher.org>
 * @author  parts by Chinamann <chinamann@users.sourceforge.net>
 * @version $Id: search.php,v 2.37 2006/02/19 16:12:48 andig2 Exp $
 */

require_once './core/session.php';
require_once './core/functions.php';
require_once './core/queryparser.php';
require_once './core/output.php';

// multiuser permission check
permission_or_die(PERM_READ);

// make sure we have an owner to filter search
if ($config['multiuser'])
{
    $all = strtoupper($lang['radio_all']);

    if (empty($owner))
    {
        if ($_SESSION['vdb']['owner'] && ($_SESSION['vdb']['owner'] == $all || check_permission(PERM_READ,getUserId($_SESSION['vdb']['owner'])))) 
        {
            $owner = $_SESSION['vdb']['owner'];
        } 
        elseif (check_permission(PERM_READ,$_COOKIE['VDBuserid'])) 
        { 
            $owner = $_COOKIE['VDBusername'];
        } 
        elseif (count($owners = out_owners(false,PERM_READ)) == 1) 
        { 
            // check if there is only one owner available
            $owner = $owners[0];
        } 
        else 
        {
            // help! take ALL
            $owner = strtoupper($lang['radio_all']);
        }   
    }

    $_SESSION['vdb']['owner'] = $owner;
}

// prepare fieldselect
$search_fields = array( 'title'         => $lang['title'],
                        'subtitle'      => $lang['subtitle'],
                        'director'      => $lang['director'],
                        'actors'        => $lang['cast'],       // differs
                        'plot'          => $lang['plot'],
                        'year'          => $lang['year'],
                        'country'       => $lang['country'],
                        'diskid'        => $lang['diskid'],
                        'owner'         => $lang['owner'],
                        'mediatype'     => $lang['mediatype'],
                        'video_codec'   => $lang['videocodec'],
                        'language'      => $lang['language'],
                        'comment'       => $lang['comment']);

// search fields to replace by physical name
$replace_fields = array('mediatype' => TBL_MEDIATYPES.'.name',
                        'owner'     => TBL_USERS.'.name',
                        'diskid'    => TBL_DATA.'.diskid');
            
// add custom fields
for ($i=1; $i<=4; $i++)
{
	if (!empty($config['custom'.$i])) 
    {
		$search_fields['custom'.$i] = $config['custom'.$i];
	}
}

// no fields given? Select all
if (empty($fields)) 
{
    // name search?
    if ($isname) {
        $fields = array('director', 'actors');
    } else {
        $fields = array_keys($search_fields);
    }
} 
// only one field? convert to array
elseif (!is_array($fields)) 
{
     $fields = array($fields);
}

// translate search fields into SQL column names, store original for template first
$smarty->assign('selected_fields', $fields);
foreach ($fields as $search_field)
{
    /*
     * field name conversion is required since the mysql AS columns can't be used in where clauses
     * add all fields here where the logical name in the template differs from the physical column name (e.g. mediatype)
     * or where the column name is ambiguous (e.g. diskid)
     */
    if (array_key_exists($search_field, $replace_fields))
    {
        if (($key = array_search($search_field, $fields)) !== false)
        {
            unset($fields[$key]);
            $fields[] = $replace_fields[$search_field];
        }        
    }        
}

// prepare search query
if (isset($q))
{
	$QUERY = '';

	if (!empty($q))
	{
		$error  = '';
		$tokens = queryparser($q, $error);

		$wild_char = (empty($nowild)) ? '%' : '';

		foreach ($tokens as $token)
		{
            // escape search token
			$token['token'] = addslashes($token['token']);

            // concatenate tokens with token operator
			$QUERY .= $token['ops'].' (';

            // concatenate all searchable fields with OR
			foreach ($fields as $field) 
            {
				$QUERY .= " ($field LIKE '$wild_char".$token['token']."$wild_char') OR ";
			}

            // concatenate custom fields with OR
			for ($custom = 1; $custom <= 4; $custom++)
			{
				if (!empty($config['custom'.$custom]) &! empty($fields['custom'.$custom])) {
					$QUERY .= " (custom$custom LIKE '$wild_char".$token['token']."$wild_char') OR ";
				}
			}
			$QUERY .= ' 1=2)';
		}
	}
	
    // filter by genres
	if (count($genres))
	{
        $TABLE  = TBL_VIDEOGENRE.',';
        $QUERY .= ' AND '.TBL_DATA.'.id = '.TBL_VIDEOGENRE.'.video_id AND (';

		foreach ($genres as $genre)
        {
            $FILTER .= 'OR '.TBL_VIDEOGENRE.'.genre_id = '.$genre.' ';
		}
		
        $FILTER = preg_replace('/^OR/', '', $FILTER);
		$QUERY .= $FILTER;
		$QUERY .= ')';
	}

    // XML / PDF export
    if ($export && $config[$export])
    {
        $func = $export.'export';
        require_once './core/'.$export.'.php';

        if (function_exists($func)) $func("1=1 $QUERY ORDER BY title, subtitle");
        exit();
    }

    // limit visibility
    if ($config['multiuser'])
    {
        if ($owner && $owner != $all)
        {
            $QUERY .= " AND ".TBL_USERS.".name = '".$owner."'";
        } 
        elseif ($owner) 
        {
        	$QUERY .= " AND ".TBL_USERS.".name IN (".owners2list(out_owners(null, PERM_READ)).")";	
        }
    }
    
    $select = 'SELECT DISTINCT '.TBL_DATA.'.id, '.TBL_DATA.'.diskid, 
                      title, subtitle, language, 
                      year, director, plot, seen, imgurl, 
                      md5, comment, disklabel, imdbID, actors, runtime, 
                      country, filename, filesize, filedate, audio_codec, 
                      video_codec, video_width, video_height, istv, 
                      lastupdate, seen, mediatype, created,
                      custom1, custom2, custom3, custom4, 
                      '.TBL_LENT.'.who, '.TBL_USERS.'.name AS owner, '.TBL_MEDIATYPES.'.name AS mediatypename
                 FROM '.$TABLE.' '.TBL_DATA.' 
            LEFT JOIN '.TBL_LENT.' ON '.TBL_DATA.'.diskid = '.TBL_LENT.'.diskid 
            LEFT JOIN '.TBL_MEDIATYPES.' ON '.TBL_DATA.'.mediatype = '.TBL_MEDIATYPES.'.id
            LEFT JOIN '.TBL_USERS.' ON owner_id = '.TBL_USERS.'.id 
                WHERE 1=1 '.$QUERY.'
             ORDER BY title, subtitle'; 

    $result = runSQL($select);

    // store query result in session for prev/next navigation
    $_SESSION['vdb']['query_result'] = array_map(create_function('&$row', 'return $row["id"];'), $result);
}

// prepare templates
tpl_page('search', $q);
tpl_list($result);

$smarty->assign('q', $q);
$smarty->assign('q_q', formvar($q));
$smarty->assign('search_fields', $search_fields);
$smarty->assign('genreselect', out_genres($genres));
$smarty->assign('engine', $config['engine']);

// person search?
if ($isname && ($config['actorpics']))
{
    $smarty->assign('imgurl', getActorThumbnail(urldecode(preg_replace('/&quot;|"/', '', formvar($q)))));
}
// docpart support
else if ($docpart)
{
    smarty_display('list.tpl');
    exit;
}

// allow XML export
if (isset($q))
{
    $link = htmlentities($_SERVER['QUERY_STRING']);     // encode for XHTML compliance
    if ($link) $link .= '&amp;';
    $link = 'search.php?'.$link;
    
    if ($config['xls']) $smarty->assign('xls', $link);
    if ($config['xml']) $smarty->assign('xml', $link);
    if ($config['pdf']) $smarty->assign('pdf', $link);
}

if ($config['multiuser'])
{
    $smarty->assign('owners', out_owners(array($all => $all), PERM_READ));
    $smarty->assign('owner', $owner);
}

// display templates
smarty_display('header.tpl');
if (!$config['http_caching']) flush();

smarty_display('search.tpl');
if (isset($q)) smarty_display('list.tpl');
smarty_display('footer.tpl');

?>