<?php
/**
 * Movie Detail View
 *
 * Shows all data of a movie
 *
 * @package videoDB
 * @author  Andreas Gohr    <a.gohr@web.de>
 * @author  Andreas Goetz   <cpuidle@gmx.de>
 * @version $Id: show.php,v 2.30 2005/09/27 12:55:18 andig2 Exp $
 */

require_once './core/functions.php';
require_once './core/custom.php';

// random view
if (empty($id)) 
{
	$count  = 0;
    $all    = strtoupper($lang['radio_all']);
    $WHERES = '';
    
    if ($config['multiuser'])
    {
        // explicit setting of owner
        if ($_SESSION['vdb']['owner'] && 
           ($_SESSION['vdb']['owner'] == $all || 
            check_permission(PERM_READ, $uid = getUserId($_SESSION['vdb']['owner'])))) 
        {
            if ($_SESSION['vdb']['owner'] == $all) $uid = -1; 
            $owner_id = $uid;
        } 
        elseif (check_permission(PERM_READ,$_COOKIE['VDBuserid'])) 
        { 
            $owner_id = $_COOKIE['VDBuserid'];
        } 
        elseif (count($owners = out_owners(false,PERM_READ,true)) == 1) 
        { 
            // check if there is only one owner available
            $owners = array_keys($owners);
            $owner_id = $owners[0];
        } 
        else 
        {
            // help! take ALL
            $owner_id = -1;
        }

        if ($owner_id && $owner_id >= 0) $_SESSION['vdb']['owner'] = getUserName($owner_id);
        else $_SESSION['vdb']['owner'] = $all;

        if ($owner_id <= 0) // All
        {
            $WHERES .= "AND owner_id IN (".owners2list(out_owners(null, PERM_READ, true), false).") ";
        } 
        else 
        {   // User
            $WHERES .= "AND owner_id = '".$owner_id."' ";
        }
    }
	
    // find a random id
    $SELECT = 'SELECT id, REVERSE(RAND(NOW())) AS rnd
                 FROM '.TBL_DATA.'
                WHERE mediatype != '.MEDIA_WISHLIST.' '.$WHERES.'
                ORDER BY rnd
                LIMIT 1';

	while (!$id)
	{
		$result = runSQL($SELECT);
        
        // prevent endless loop
        if (!count($result) || ($count++ > 50)) break;
        
		$id = $result[0]['id'];
		if (!adultcheck($id)) $id = 0;  //adult movie? -> try again
	}
    
	// id still empty? go back to index.
	if (empty($id)) redirect('index.php');
}

// get data (id may be empty on a empty database)
if (!empty($id))
{
	// no adult permissions? -> back to index
	if (!adultcheck($id)) redirect('index.php');

    // XML export?
    if (($export == 'xml') && $config['xml'])
    {
        require_once './core/'.$export.'.php';
        xmlexport('videodata.id = '.$id);
        exit;
    }

    $SELECT = 'SELECT '.TBL_DATA.'.id, title, subtitle, language, diskid, comment,
                      disklabel, imdbID, year, imgurl, director, actors, runtime,  
                      country, plot, filename, filesize, filedate, audio_codec,  
                      video_codec, video_width, video_height, istv, lastupdate,  
                      seen, email, custom1, custom2, custom3, custom4,   
                      '.TBL_USERS.'.name AS owner, '.TBL_MEDIATYPES.'.name AS mediatype  
                 FROM '.TBL_DATA.'   
            LEFT JOIN '.TBL_USERS.' ON owner_id = '.TBL_USERS.'.id  
            LEFT JOIN '.TBL_MEDIATYPES.' ON mediatype = '.TBL_MEDIATYPES.'.id  
                WHERE '.TBL_DATA.'.id = '.$id;  
	$res   = runSQL($SELECT);

    // existing id?
    if (!count($res)) redirect('index.php');
    
    // get the item
    $video = $res[0];

	// get seen status for currently logged in user
	$video['seen'] = get_userseen($id, $video['seen']);

    // is it editable/ copyable?
    $video['editable'] = localnet();
    $video['copyable'] = localnet();
    
    // multi-user permissions
    if ($config['multiuser'])
    {
        $video['editable'] = $video['editable'] && check_permission(PERM_WRITE, getUserId($video['owner']));

        if (check_permission(PERM_WRITE, $_COOKIE['VDBuserid']) || 
            count(out_owners(false, PERM_WRITE)) == 1) 
        {
            $video['copyable'] = true;
		}
        else 
        {
            // don't know where to copy!
            $video['copyable'] = false;	
        }
    }

    // save if seen was clicked
    if ($save)
    {
        if (!$seen) $seen = 0;
        if ($video['editable'])
        {
            // owner can save directly to videodata
            runSQL('UPDATE '.TBL_DATA.' SET seen = '.$seen.' WHERE id = '.$id);
        }
        set_userseen($id, $seen); // additonally save to userseen
        $video['seen'] = $seen;
    }

    // diskid to global scope:
    $diskid = $video['diskid'];

    // check if it is lent and to whom
    $SELECT = 'SELECT who
                 FROM '.TBL_LENT."
                WHERE diskid = '".addslashes($diskid)."'";
    $result = runSQL($SELECT);

    if (isset($result[0]['who']))
    {
        $video['who'] = $result[0]['who'];
    }

/*    
    // append child episodes
    $SELECT = 'SELECT *
                 FROM '.TBL_DATA.'                       
                WHERE '.TBL_DATA.'.parent_id = '.$id.'                      
             ORDER BY season, episode, title, subtitle';                      
    $episodes = runSQL($SELECT);
    
    $video['episodes'] = $episodes;
*/

    // previous/next buttons
    $ids = $_SESSION['vdb']['query_result'];
    if (is_array($ids))
    {
        if (($key = array_search($id, $ids)) !== false)
        {
            $video['prev_id'] = ($key > 0) ? $ids[$key-1] : 0;
            $video['next_id'] = ($key < count($ids)-1) ? $ids[$key+1] : 0;
        }
    }
}

// prepare templates
tpl_page('detailview', $video['title']);
if (!empty($id)) tpl_show($video);

// caching enabled?
if ($config['http_caching'])
{
    require_once('./core/httpcache.php');
    httpCacheCaptureStart();
}

// display templates
smarty_display('header.tpl');
if (!$config['http_caching']) flush();
if (!empty($id)) smarty_display('show.tpl', $id);
smarty_display('footer.tpl');

// caching enabled?
if ($config['http_caching'])
{
    httpCacheOutput('show'.$id, httpCacheCaptureEnd());
}

?>
