<?php
/**
 * Statistics
 *
 * Shows general statistics about the database
 *
 * @package videoDB
 * @author  Andreas Gohr <a.gohr@web.de>
 * @version $Id: stats.php,v 2.14 2005/06/28 14:10:13 chinamann Exp $
 */

require_once './core/functions.php';

/*
 * Helper function for comparing an associative array by it's 'count' values
 */
function compare_count($a, $b)
{
    return strnatcasecmp($b['count'], $a['count']); 
}

/*
 * De-uniquify multi-language statistics
 *
 * Makes sure videos with mulitple languages are count once for each language
 * contained instead of once for a unique combination
 *
 * @param   array   $langs  array of all languages from 
 */ 
function collapse_multiple_languages($langs) 
{
    $templangs = array();
    
    foreach($langs as $key => $val) 
    {
        $lang_ary  = preg_split('/,\s*/', $val['language']);
        foreach($lang_ary as $numid => $lang) 
        {
            // take care of english vs. English
            $lang = ucwords($lang);
            
            $templangs[$lang]['language'] = $lang;
            $templangs[$lang]['count']   += $val['count'];
        }
    }
    usort($templangs, 'compare_count');
    
    return $templangs;
}

$all = strtoupper($lang['radio_all']);
if (!$config['multiuser']) 
{
    $owner_id = -1;
}

if (empty($owner_id)) 
{
    // explicit setting of owner
    if ($_SESSION['vdb']['owner'] && ($_SESSION['vdb']['owner'] == $all || check_permission(PERM_READ,$uid = getUserId($_SESSION['vdb']['owner'])))) {
        if($_SESSION['vdb']['owner'] == $all) $uid = -1; 
            $owner_id = $uid;
    } 
    elseif (check_permission(PERM_READ,$_COOKIE['VDBuserid'])) 
    { 
        $owner_id = $_COOKIE['VDBuserid'];
    } 
    elseif (count($owners = out_owners(false,PERM_READ,true)) == 1) 
    { 
        // check if there is only one owner available
        $owners = array_keys($owners);
        $owner_id = $owners[0];
    } 
    else 
    {
        // help! take ALL
        $owner_id = -1;
    }
}

if ($owner_id && $owner_id >= 0) $_SESSION['vdb']['owner'] = getUserName($owner_id);
else $_SESSION['vdb']['owner'] = $all;

if ($config['multiuser']) 
{
    if ($owner_id < 0) // All
    {
        $WHERES .= " AND owner_id IN (".owners2list(out_owners(null, PERM_READ, true), false).")";
    } 
    else 
    {   // User
        $WHERES .= " AND owner_id = '".$owner_id."'";
    }
}

$result = runSQL('SELECT COUNT(*) AS count FROM '.TBL_DATA.' WHERE 1=1'.$WHERES);
$stats['count_all']  = $result[0]['count'];

$result = runSQL('SELECT COUNT(*) AS count FROM '.TBL_DATA.' WHERE istv = 1'.$WHERES);
$stats['count_tv']   = $result[0]['count'];

$result = runSQL('SELECT DISTINCT diskid FROM '.TBL_DATA.' WHERE 1=1'.$WHERES);
$stats['count_disk'] = count($result);

$result = runSQL('SELECT AVG(filesize) AS afs FROM '.TBL_DATA.' WHERE filesize > 0'.$WHERES);
$stats['avg_size']   = round(($result[0]['afs'])/(1024*1024), 2);

$result = runSQL('SELECT SUM(filesize) AS sfs,
                         AVG(runtime)  AS art,
                         SUM(runtime)  AS srt
                  FROM '.TBL_DATA.' WHERE 1=1'.$WHERES);
$stats['sum_size']   = round(($result[0]['sfs'])/(1024*1024*1024), 2);
$stats['avg_time']   = round($result[0]['art'], 2);
$stats['sum_time']   = round(($result[0]['srt'])/60, 2);

$result = runSQL('SELECT SUM(runtime) AS srt FROM '.TBL_DATA.' where seen=1'.$WHERES);
$stats['seen_time']  = round($result[0]['srt']/60, 2);  

$result = runSQL('SELECT A.name, COUNT(*) AS count, A.id
                    FROM '.TBL_GENRES.' A, '.TBL_VIDEOGENRE.' B, '.TBL_DATA.' C 
                   WHERE B.genre_id = A.id
                   	 AND B.video_id = C.id'.$WHERES.'
                GROUP BY A.name
                ORDER BY count DESC');
$stats['count_genre'] = $result;

$result = runSQL('SELECT A.name, COUNT(*) AS count, A.id
                    FROM '.TBL_MEDIATYPES.' A, '.TBL_DATA.' B
                   WHERE B.mediatype = A.id'.$WHERES.' 
                GROUP BY A.name
                ORDER BY count DESC');
$stats['count_media'] = $result;

$result = runSQL('SELECT language, COUNT(*) AS count
                    FROM '.TBL_DATA.' WHERE 1=1'.$WHERES.' 
                GROUP BY language
                ORDER BY count DESC');
$stats['count_lang'] = collapse_multiple_languages($result);
                     
$result = runSQL('SELECT video_codec, COUNT(*) AS count
                    FROM '.TBL_DATA.' WHERE 1=1'.$WHERES.' 
                GROUP BY video_codec
                ORDER BY count DESC');
$stats['count_vcodec'] = $result;

// year statistics
$result = runSQL('SELECT year, COUNT(*) AS count
                    FROM '.TBL_DATA.'
                   WHERE year > 0'.$WHERES.' 
                GROUP BY year
                ORDER BY year');
$minyear = $result[0]['year'];
$maxyear = $result[count($result)-1]['year'];

for ($i = $minyear; $i <= $maxyear; $i++)
{
    $years[$i] = 0;
}

$maxcount = 0;
if (is_array($result))
{
	foreach ($result AS $year)
	{
		$years[$year['year']] = $year['count'];
        if ($year['count'] > $maxcount) $maxcount = $year['count'];
	}
}

$stats['count_year'] = $years;
$stats['first_year'] = $minyear;
$stats['last_year']  = $maxyear;
$stats['max_count']  = $maxcount;

// prepare templates
tpl_page();

$smarty->assign('stats', $stats);
if ($config['multiuser'])
{
    $smarty->assign('owners', out_owners(array('-1' => $all),PERM_READ,true));
    $smarty->assign('owner_id', $owner_id);
}

// display templates
tpl_display('stats.tpl');

?>
